from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset







T = TypeVar("T", bound="AuthMetadataQuotaEnabledServers")



@_attrs_define
class AuthMetadataQuotaEnabledServers:
    """ 
        Attributes:
            quota (float):
            current (float):
            remaining (float):
     """

    quota: float
    current: float
    remaining: float





    def to_dict(self) -> dict[str, Any]:
        quota = self.quota

        current = self.current

        remaining = self.remaining


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "quota": quota,
            "current": current,
            "remaining": remaining,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        quota = d.pop("quota")

        current = d.pop("current")

        remaining = d.pop("remaining")

        auth_metadata_quota_enabled_servers = cls(
            quota=quota,
            current=current,
            remaining=remaining,
        )

        return auth_metadata_quota_enabled_servers




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.team_with_member_count import TeamWithMemberCount





T = TypeVar("T", bound="GetTeamsResponse200")



@_attrs_define
class GetTeamsResponse200:
    """ 
        Attributes:
            teams (list[TeamWithMemberCount]):
     """

    teams: list[TeamWithMemberCount]





    def to_dict(self) -> dict[str, Any]:
        from ..models.team_with_member_count import TeamWithMemberCount
        teams = []
        for teams_item_data in self.teams:
            teams_item = teams_item_data.to_dict()
            teams.append(teams_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "teams": teams,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.team_with_member_count import TeamWithMemberCount
        d = dict(src_dict)
        teams = []
        _teams = d.pop("teams")
        for teams_item_data in (_teams):
            teams_item = TeamWithMemberCount.from_dict(teams_item_data)



            teams.append(teams_item)


        get_teams_response_200 = cls(
            teams=teams,
        )

        return get_teams_response_200




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.user_identity_type import UserIdentityType






T = TypeVar("T", bound="UserIdentity")



@_attrs_define
class UserIdentity:
    """ 
        Attributes:
            tenant_id (float):
            external_id (str):
            user_id (float):
            type_ (UserIdentityType):
            created_at (str):
            updated_at (str):
     """

    tenant_id: float
    external_id: str
    user_id: float
    type_: UserIdentityType
    created_at: str
    updated_at: str





    def to_dict(self) -> dict[str, Any]:
        tenant_id = self.tenant_id

        external_id = self.external_id

        user_id = self.user_id

        type_ = self.type_.value

        created_at = self.created_at

        updated_at = self.updated_at


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "tenantId": tenant_id,
            "externalId": external_id,
            "userId": user_id,
            "type": type_,
            "createdAt": created_at,
            "updatedAt": updated_at,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        tenant_id = d.pop("tenantId")

        external_id = d.pop("externalId")

        user_id = d.pop("userId")

        type_ = UserIdentityType(d.pop("type"))




        created_at = d.pop("createdAt")

        updated_at = d.pop("updatedAt")

        user_identity = cls(
            tenant_id=tenant_id,
            external_id=external_id,
            user_id=user_id,
            type_=type_,
            created_at=created_at,
            updated_at=updated_at,
        )

        return user_identity




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Literal, cast






T = TypeVar("T", bound="ServerCredentialsApiKeys")



@_attrs_define
class ServerCredentialsApiKeys:
    """ 
        Attributes:
            type_ (Literal['apikey']):
            apikeys (list[list[str]]):
     """

    type_: Literal['apikey']
    apikeys: list[list[str]]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        apikeys = []
        for apikeys_item_data in self.apikeys:
            apikeys_item = []
            for apikeys_item_item_data in apikeys_item_data:
                apikeys_item_item: str
                apikeys_item_item = apikeys_item_item_data
                apikeys_item.append(apikeys_item_item)


            apikeys.append(apikeys_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "apikeys": apikeys,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = cast(Literal['apikey'] , d.pop("type"))
        if type_ != 'apikey':
            raise ValueError(f"type must match const 'apikey', got '{type_}'")

        apikeys = []
        _apikeys = d.pop("apikeys")
        for apikeys_item_data in (_apikeys):
            apikeys_item = []
            _apikeys_item = apikeys_item_data
            for apikeys_item_item_data in (_apikeys_item):
                def _parse_apikeys_item_item(data: object) -> str:
                    return cast(str, data)

                apikeys_item_item = _parse_apikeys_item_item(apikeys_item_item_data)

                apikeys_item.append(apikeys_item_item)

            apikeys.append(apikeys_item)


        server_credentials_api_keys = cls(
            type_=type_,
            apikeys=apikeys,
        )


        server_credentials_api_keys.additional_properties = d
        return server_credentials_api_keys

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Literal, cast






T = TypeVar("T", bound="ServerFile")



@_attrs_define
class ServerFile:
    """ 
        Attributes:
            tenant_id (str):
            server_id (str):
            id (str):
            filename (str):
            source_type (Literal['aws-secrets-manager'] | Literal['aws-ssm'] | Literal['gcs']):
            source_id (str):
            size (float | None):
            credential_id (None | str):
            created_at (str):
            updated_at (str):
     """

    tenant_id: str
    server_id: str
    id: str
    filename: str
    source_type: Literal['aws-secrets-manager'] | Literal['aws-ssm'] | Literal['gcs']
    source_id: str
    size: float | None
    credential_id: None | str
    created_at: str
    updated_at: str





    def to_dict(self) -> dict[str, Any]:
        tenant_id = self.tenant_id

        server_id = self.server_id

        id = self.id

        filename = self.filename

        source_type: Literal['aws-secrets-manager'] | Literal['aws-ssm'] | Literal['gcs']
        source_type = self.source_type

        source_id = self.source_id

        size: float | None
        size = self.size

        credential_id: None | str
        credential_id = self.credential_id

        created_at = self.created_at

        updated_at = self.updated_at


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "tenantId": tenant_id,
            "serverId": server_id,
            "id": id,
            "filename": filename,
            "sourceType": source_type,
            "sourceId": source_id,
            "size": size,
            "credentialId": credential_id,
            "createdAt": created_at,
            "updatedAt": updated_at,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        tenant_id = d.pop("tenantId")

        server_id = d.pop("serverId")

        id = d.pop("id")

        filename = d.pop("filename")

        def _parse_source_type(data: object) -> Literal['aws-secrets-manager'] | Literal['aws-ssm'] | Literal['gcs']:
            source_type_type_0 = cast(Literal['gcs'] , data)
            if source_type_type_0 != 'gcs':
                raise ValueError(f"sourceType_type_0 must match const 'gcs', got '{source_type_type_0}'")
            return source_type_type_0
            source_type_type_1 = cast(Literal['aws-ssm'] , data)
            if source_type_type_1 != 'aws-ssm':
                raise ValueError(f"sourceType_type_1 must match const 'aws-ssm', got '{source_type_type_1}'")
            return source_type_type_1
            source_type_type_2 = cast(Literal['aws-secrets-manager'] , data)
            if source_type_type_2 != 'aws-secrets-manager':
                raise ValueError(f"sourceType_type_2 must match const 'aws-secrets-manager', got '{source_type_type_2}'")
            return source_type_type_2

        source_type = _parse_source_type(d.pop("sourceType"))


        source_id = d.pop("sourceId")

        def _parse_size(data: object) -> float | None:
            if data is None:
                return data
            return cast(float | None, data)

        size = _parse_size(d.pop("size"))


        def _parse_credential_id(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        credential_id = _parse_credential_id(d.pop("credentialId"))


        created_at = d.pop("createdAt")

        updated_at = d.pop("updatedAt")

        server_file = cls(
            tenant_id=tenant_id,
            server_id=server_id,
            id=id,
            filename=filename,
            source_type=source_type,
            source_id=source_id,
            size=size,
            credential_id=credential_id,
            created_at=created_at,
            updated_at=updated_at,
        )

        return server_file




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import Literal, cast






T = TypeVar("T", bound="HashiCorpVaultConfigurationOutput")



@_attrs_define
class HashiCorpVaultConfigurationOutput:
    """ 
        Attributes:
            type_ (Literal['hashicorp_vault']):
            address (str):
            token (str):
            mount_path (str):  Default: 'secret'.
            namespace (str | Unset):
     """

    type_: Literal['hashicorp_vault']
    address: str
    token: str
    mount_path: str = 'secret'
    namespace: str | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        address = self.address

        token = self.token

        mount_path = self.mount_path

        namespace = self.namespace


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "type": type_,
            "address": address,
            "token": token,
            "mountPath": mount_path,
        })
        if namespace is not UNSET:
            field_dict["namespace"] = namespace

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = cast(Literal['hashicorp_vault'] , d.pop("type"))
        if type_ != 'hashicorp_vault':
            raise ValueError(f"type must match const 'hashicorp_vault', got '{type_}'")

        address = d.pop("address")

        token = d.pop("token")

        mount_path = d.pop("mountPath")

        namespace = d.pop("namespace", UNSET)

        hashi_corp_vault_configuration_output = cls(
            type_=type_,
            address=address,
            token=token,
            mount_path=mount_path,
            namespace=namespace,
        )

        return hashi_corp_vault_configuration_output




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="UpdateServerRequestOauthMetadataType0AsType0")



@_attrs_define
class UpdateServerRequestOauthMetadataType0AsType0:
    """ 
        Attributes:
            issuer (str | Unset):  Default: ''.
            authorize_endpoint (str | Unset):  Default: ''.
            device_authorization_endpoint (str | Unset):  Default: ''.
            token_endpoint (str | Unset):  Default: ''.
            extra_authorization_parameters (str | Unset):  Default: ''.
            registration_endpoint (str | Unset):
            supports_pkce (bool | Unset):  Default: False.
            supported_client_auth_methods (list[str] | Unset):
            supports_dynamic_client_registration (bool | Unset):  Default: False.
            no_dynamic_client_registration_reason (None | str | Unset):
            service_documentation (str | Unset):  Default: ''.
            metadata_url (str | Unset):  Default: ''.
            available_scopes (list[str] | Unset):
     """

    issuer: str | Unset = ''
    authorize_endpoint: str | Unset = ''
    device_authorization_endpoint: str | Unset = ''
    token_endpoint: str | Unset = ''
    extra_authorization_parameters: str | Unset = ''
    registration_endpoint: str | Unset = UNSET
    supports_pkce: bool | Unset = False
    supported_client_auth_methods: list[str] | Unset = UNSET
    supports_dynamic_client_registration: bool | Unset = False
    no_dynamic_client_registration_reason: None | str | Unset = UNSET
    service_documentation: str | Unset = ''
    metadata_url: str | Unset = ''
    available_scopes: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        issuer = self.issuer

        authorize_endpoint = self.authorize_endpoint

        device_authorization_endpoint = self.device_authorization_endpoint

        token_endpoint = self.token_endpoint

        extra_authorization_parameters = self.extra_authorization_parameters

        registration_endpoint = self.registration_endpoint

        supports_pkce = self.supports_pkce

        supported_client_auth_methods: list[str] | Unset = UNSET
        if not isinstance(self.supported_client_auth_methods, Unset):
            supported_client_auth_methods = self.supported_client_auth_methods



        supports_dynamic_client_registration = self.supports_dynamic_client_registration

        no_dynamic_client_registration_reason: None | str | Unset
        if isinstance(self.no_dynamic_client_registration_reason, Unset):
            no_dynamic_client_registration_reason = UNSET
        else:
            no_dynamic_client_registration_reason = self.no_dynamic_client_registration_reason

        service_documentation = self.service_documentation

        metadata_url = self.metadata_url

        available_scopes: list[str] | Unset = UNSET
        if not isinstance(self.available_scopes, Unset):
            available_scopes = self.available_scopes




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if issuer is not UNSET:
            field_dict["issuer"] = issuer
        if authorize_endpoint is not UNSET:
            field_dict["authorizeEndpoint"] = authorize_endpoint
        if device_authorization_endpoint is not UNSET:
            field_dict["deviceAuthorizationEndpoint"] = device_authorization_endpoint
        if token_endpoint is not UNSET:
            field_dict["tokenEndpoint"] = token_endpoint
        if extra_authorization_parameters is not UNSET:
            field_dict["extraAuthorizationParameters"] = extra_authorization_parameters
        if registration_endpoint is not UNSET:
            field_dict["registrationEndpoint"] = registration_endpoint
        if supports_pkce is not UNSET:
            field_dict["supportsPKCE"] = supports_pkce
        if supported_client_auth_methods is not UNSET:
            field_dict["supportedClientAuthMethods"] = supported_client_auth_methods
        if supports_dynamic_client_registration is not UNSET:
            field_dict["supportsDynamicClientRegistration"] = supports_dynamic_client_registration
        if no_dynamic_client_registration_reason is not UNSET:
            field_dict["noDynamicClientRegistrationReason"] = no_dynamic_client_registration_reason
        if service_documentation is not UNSET:
            field_dict["serviceDocumentation"] = service_documentation
        if metadata_url is not UNSET:
            field_dict["metadataUrl"] = metadata_url
        if available_scopes is not UNSET:
            field_dict["availableScopes"] = available_scopes

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        issuer = d.pop("issuer", UNSET)

        authorize_endpoint = d.pop("authorizeEndpoint", UNSET)

        device_authorization_endpoint = d.pop("deviceAuthorizationEndpoint", UNSET)

        token_endpoint = d.pop("tokenEndpoint", UNSET)

        extra_authorization_parameters = d.pop("extraAuthorizationParameters", UNSET)

        registration_endpoint = d.pop("registrationEndpoint", UNSET)

        supports_pkce = d.pop("supportsPKCE", UNSET)

        supported_client_auth_methods = cast(list[str], d.pop("supportedClientAuthMethods", UNSET))


        supports_dynamic_client_registration = d.pop("supportsDynamicClientRegistration", UNSET)

        def _parse_no_dynamic_client_registration_reason(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        no_dynamic_client_registration_reason = _parse_no_dynamic_client_registration_reason(d.pop("noDynamicClientRegistrationReason", UNSET))


        service_documentation = d.pop("serviceDocumentation", UNSET)

        metadata_url = d.pop("metadataUrl", UNSET)

        available_scopes = cast(list[str], d.pop("availableScopes", UNSET))


        update_server_request_oauth_metadata_type_0_as_type_0 = cls(
            issuer=issuer,
            authorize_endpoint=authorize_endpoint,
            device_authorization_endpoint=device_authorization_endpoint,
            token_endpoint=token_endpoint,
            extra_authorization_parameters=extra_authorization_parameters,
            registration_endpoint=registration_endpoint,
            supports_pkce=supports_pkce,
            supported_client_auth_methods=supported_client_auth_methods,
            supports_dynamic_client_registration=supports_dynamic_client_registration,
            no_dynamic_client_registration_reason=no_dynamic_client_registration_reason,
            service_documentation=service_documentation,
            metadata_url=metadata_url,
            available_scopes=available_scopes,
        )


        update_server_request_oauth_metadata_type_0_as_type_0.additional_properties = d
        return update_server_request_oauth_metadata_type_0_as_type_0

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.auth_metadata_tenant_abilities import AuthMetadataTenantAbilities


T = TypeVar("T", bound="AuthMetadataTenant")


@_attrs_define
class AuthMetadataTenant:
    """
    Attributes:
        id (str):
        is_mcp_authorization_api_key_enabled (bool):
        grant_users_install_servers_policy (bool):
        display_name (str):
        is_trial (bool):
        subscription_plan (str):
        subscription_seats (float):
        free_credits (float):
        paid_credits (float):
        abilities (AuthMetadataTenantAbilities):
        number_of_enabled_users (float | Unset):
    """

    id: str
    is_mcp_authorization_api_key_enabled: bool
    grant_users_install_servers_policy: bool
    display_name: str
    is_trial: bool
    subscription_plan: str
    subscription_seats: float
    free_credits: float
    paid_credits: float
    abilities: AuthMetadataTenantAbilities
    number_of_enabled_users: float | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        is_mcp_authorization_api_key_enabled = self.is_mcp_authorization_api_key_enabled

        grant_users_install_servers_policy = self.grant_users_install_servers_policy

        display_name = self.display_name

        is_trial = self.is_trial

        subscription_plan = self.subscription_plan

        subscription_seats = self.subscription_seats

        free_credits = self.free_credits

        paid_credits = self.paid_credits

        abilities = self.abilities.to_dict()

        number_of_enabled_users = self.number_of_enabled_users

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "isMcpAuthorizationApiKeyEnabled": is_mcp_authorization_api_key_enabled,
                "grantUsersInstallServersPolicy": grant_users_install_servers_policy,
                "displayName": display_name,
                "isTrial": is_trial,
                "subscriptionPlan": subscription_plan,
                "subscriptionSeats": subscription_seats,
                "freeCredits": free_credits,
                "paidCredits": paid_credits,
                "abilities": abilities,
            }
        )
        if number_of_enabled_users is not UNSET:
            field_dict["numberOfEnabledUsers"] = number_of_enabled_users

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.auth_metadata_tenant_abilities import AuthMetadataTenantAbilities

        d = dict(src_dict)
        id = d.pop("id")

        is_mcp_authorization_api_key_enabled = d.pop("isMcpAuthorizationApiKeyEnabled")

        grant_users_install_servers_policy = d.pop("grantUsersInstallServersPolicy")

        display_name = d.pop("displayName")

        is_trial = d.pop("isTrial")

        subscription_plan = d.pop("subscriptionPlan")

        subscription_seats = d.pop("subscriptionSeats")

        free_credits = d.pop("freeCredits")

        paid_credits = d.pop("paidCredits")

        abilities = AuthMetadataTenantAbilities.from_dict(d.pop("abilities"))

        number_of_enabled_users = d.pop("numberOfEnabledUsers", UNSET)

        auth_metadata_tenant = cls(
            id=id,
            is_mcp_authorization_api_key_enabled=is_mcp_authorization_api_key_enabled,
            grant_users_install_servers_policy=grant_users_install_servers_policy,
            display_name=display_name,
            is_trial=is_trial,
            subscription_plan=subscription_plan,
            subscription_seats=subscription_seats,
            free_credits=free_credits,
            paid_credits=paid_credits,
            abilities=abilities,
            number_of_enabled_users=number_of_enabled_users,
        )

        return auth_metadata_tenant



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.get_teams_team_id_members_response_200_members_item import GetTeamsTeamIdMembersResponse200MembersItem





T = TypeVar("T", bound="GetTeamsTeamIdMembersResponse200")



@_attrs_define
class GetTeamsTeamIdMembersResponse200:
    """ 
        Attributes:
            members (list[GetTeamsTeamIdMembersResponse200MembersItem]):
     """

    members: list[GetTeamsTeamIdMembersResponse200MembersItem]





    def to_dict(self) -> dict[str, Any]:
        from ..models.get_teams_team_id_members_response_200_members_item import GetTeamsTeamIdMembersResponse200MembersItem
        members = []
        for members_item_data in self.members:
            members_item = members_item_data.to_dict()
            members.append(members_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "members": members,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_teams_team_id_members_response_200_members_item import GetTeamsTeamIdMembersResponse200MembersItem
        d = dict(src_dict)
        members = []
        _members = d.pop("members")
        for members_item_data in (_members):
            members_item = GetTeamsTeamIdMembersResponse200MembersItem.from_dict(members_item_data)



            members.append(members_item)


        get_teams_team_id_members_response_200 = cls(
            members=members,
        )

        return get_teams_team_id_members_response_200




from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.server_o_auth_metadata_as import ServerOAuthMetadataAs
    from ..models.server_o_auth_metadata_resource import ServerOAuthMetadataResource


T = TypeVar("T", bound="ServerOAuthMetadata")


@_attrs_define
class ServerOAuthMetadata:
    """
    Attributes:
        requires_authorization (bool):
        resource (ServerOAuthMetadataResource | Unset):
        as_ (ServerOAuthMetadataAs | Unset):
    """

    requires_authorization: bool
    resource: ServerOAuthMetadataResource | Unset = UNSET
    as_: ServerOAuthMetadataAs | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        requires_authorization = self.requires_authorization

        resource: dict[str, Any] | Unset = UNSET
        if not isinstance(self.resource, Unset):
            resource = self.resource.to_dict()

        as_: dict[str, Any] | Unset = UNSET
        if not isinstance(self.as_, Unset):
            as_ = self.as_.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "requiresAuthorization": requires_authorization,
            }
        )
        if resource is not UNSET:
            field_dict["resource"] = resource
        if as_ is not UNSET:
            field_dict["as"] = as_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.server_o_auth_metadata_as import ServerOAuthMetadataAs
        from ..models.server_o_auth_metadata_resource import ServerOAuthMetadataResource

        d = dict(src_dict)
        requires_authorization = d.pop("requiresAuthorization")

        _resource = d.pop("resource", UNSET)
        resource: ServerOAuthMetadataResource | Unset
        if isinstance(_resource, Unset):
            resource = UNSET
        else:
            resource = ServerOAuthMetadataResource.from_dict(_resource)

        _as_ = d.pop("as", UNSET)
        as_: ServerOAuthMetadataAs | Unset
        if isinstance(_as_, Unset):
            as_ = UNSET
        else:
            as_ = ServerOAuthMetadataAs.from_dict(_as_)

        server_o_auth_metadata = cls(
            requires_authorization=requires_authorization,
            resource=resource,
            as_=as_,
        )

        return server_o_auth_metadata



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.schema_4 import Schema4
from ..types import UNSET, Unset
from typing import cast
from typing import Literal, cast






T = TypeVar("T", bound="PutMcpServersServerSlugAuthorizationBody")



@_attrs_define
class PutMcpServersServerSlugAuthorizationBody:
    """ 
        Attributes:
            method (Schema4):
            credentials_scope (Literal['server'] | Literal['user']):
            apikeys (list[str] | Unset):
     """

    method: Schema4
    credentials_scope: Literal['server'] | Literal['user']
    apikeys: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        method = self.method.value

        credentials_scope: Literal['server'] | Literal['user']
        credentials_scope = self.credentials_scope

        apikeys: list[str] | Unset = UNSET
        if not isinstance(self.apikeys, Unset):
            apikeys = self.apikeys




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "method": method,
            "credentialsScope": credentials_scope,
        })
        if apikeys is not UNSET:
            field_dict["apikeys"] = apikeys

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        method = Schema4(d.pop("method"))




        def _parse_credentials_scope(data: object) -> Literal['server'] | Literal['user']:
            componentsschemas_schema5_type_0 = cast(Literal['server'] , data)
            if componentsschemas_schema5_type_0 != 'server':
                raise ValueError(f"/components/schemas/__schema5_type_0 must match const 'server', got '{componentsschemas_schema5_type_0}'")
            return componentsschemas_schema5_type_0
            componentsschemas_schema5_type_1 = cast(Literal['user'] , data)
            if componentsschemas_schema5_type_1 != 'user':
                raise ValueError(f"/components/schemas/__schema5_type_1 must match const 'user', got '{componentsschemas_schema5_type_1}'")
            return componentsschemas_schema5_type_1

        credentials_scope = _parse_credentials_scope(d.pop("credentialsScope"))


        apikeys = cast(list[str], d.pop("apikeys", UNSET))


        put_mcp_servers_server_slug_authorization_body = cls(
            method=method,
            credentials_scope=credentials_scope,
            apikeys=apikeys,
        )


        put_mcp_servers_server_slug_authorization_body.additional_properties = d
        return put_mcp_servers_server_slug_authorization_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.secret_mapping_response import SecretMappingResponse





T = TypeVar("T", bound="SecretMappingListResponse")



@_attrs_define
class SecretMappingListResponse:
    """ 
        Attributes:
            mappings (list[SecretMappingResponse]):
     """

    mappings: list[SecretMappingResponse]





    def to_dict(self) -> dict[str, Any]:
        from ..models.secret_mapping_response import SecretMappingResponse
        mappings = []
        for mappings_item_data in self.mappings:
            mappings_item = mappings_item_data.to_dict()
            mappings.append(mappings_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "mappings": mappings,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.secret_mapping_response import SecretMappingResponse
        d = dict(src_dict)
        mappings = []
        _mappings = d.pop("mappings")
        for mappings_item_data in (_mappings):
            mappings_item = SecretMappingResponse.from_dict(mappings_item_data)



            mappings.append(mappings_item)


        secret_mapping_list_response = cls(
            mappings=mappings,
        )

        return secret_mapping_list_response




from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

from ..models.schema_76 import Schema76

if TYPE_CHECKING:
    from ..models.aws_secrets_manager_configuration_output import AwsSecretsManagerConfigurationOutput
    from ..models.azure_key_vault_configuration_output import AzureKeyVaultConfigurationOutput
    from ..models.gcp_secret_manager_configuration_output import GcpSecretManagerConfigurationOutput
    from ..models.hashi_corp_vault_configuration_output import HashiCorpVaultConfigurationOutput
    from ..models.infisical_configuration_output import InfisicalConfigurationOutput


T = TypeVar("T", bound="SecretStoreDetailResponse")


@_attrs_define
class SecretStoreDetailResponse:
    """
    Attributes:
        id (str):
        name (str):
        type_ (Schema76):
        is_enabled (bool):
        created_at (str):
        updated_at (str):
        configuration (AwsSecretsManagerConfigurationOutput | AzureKeyVaultConfigurationOutput |
            GcpSecretManagerConfigurationOutput | HashiCorpVaultConfigurationOutput | InfisicalConfigurationOutput):
    """

    id: str
    name: str
    type_: Schema76
    is_enabled: bool
    created_at: str
    updated_at: str
    configuration: (
        AwsSecretsManagerConfigurationOutput
        | AzureKeyVaultConfigurationOutput
        | GcpSecretManagerConfigurationOutput
        | HashiCorpVaultConfigurationOutput
        | InfisicalConfigurationOutput
    )

    def to_dict(self) -> dict[str, Any]:
        from ..models.aws_secrets_manager_configuration_output import AwsSecretsManagerConfigurationOutput
        from ..models.gcp_secret_manager_configuration_output import GcpSecretManagerConfigurationOutput
        from ..models.hashi_corp_vault_configuration_output import HashiCorpVaultConfigurationOutput
        from ..models.infisical_configuration_output import InfisicalConfigurationOutput

        id = self.id

        name = self.name

        type_ = self.type_.value

        is_enabled = self.is_enabled

        created_at = self.created_at

        updated_at = self.updated_at

        configuration: dict[str, Any]
        if isinstance(self.configuration, AwsSecretsManagerConfigurationOutput):
            configuration = self.configuration.to_dict()
        elif isinstance(self.configuration, GcpSecretManagerConfigurationOutput):
            configuration = self.configuration.to_dict()
        elif isinstance(self.configuration, HashiCorpVaultConfigurationOutput):
            configuration = self.configuration.to_dict()
        elif isinstance(self.configuration, InfisicalConfigurationOutput):
            configuration = self.configuration.to_dict()
        else:
            configuration = self.configuration.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
                "type": type_,
                "isEnabled": is_enabled,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "configuration": configuration,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.aws_secrets_manager_configuration_output import AwsSecretsManagerConfigurationOutput
        from ..models.azure_key_vault_configuration_output import AzureKeyVaultConfigurationOutput
        from ..models.gcp_secret_manager_configuration_output import GcpSecretManagerConfigurationOutput
        from ..models.hashi_corp_vault_configuration_output import HashiCorpVaultConfigurationOutput
        from ..models.infisical_configuration_output import InfisicalConfigurationOutput

        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        type_ = Schema76(d.pop("type"))

        is_enabled = d.pop("isEnabled")

        created_at = d.pop("createdAt")

        updated_at = d.pop("updatedAt")

        def _parse_configuration(
            data: object,
        ) -> (
            AwsSecretsManagerConfigurationOutput
            | AzureKeyVaultConfigurationOutput
            | GcpSecretManagerConfigurationOutput
            | HashiCorpVaultConfigurationOutput
            | InfisicalConfigurationOutput
        ):
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                configuration_type_0 = AwsSecretsManagerConfigurationOutput.from_dict(data)

                return configuration_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                configuration_type_1 = GcpSecretManagerConfigurationOutput.from_dict(data)

                return configuration_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                configuration_type_2 = HashiCorpVaultConfigurationOutput.from_dict(data)

                return configuration_type_2
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                configuration_type_3 = InfisicalConfigurationOutput.from_dict(data)

                return configuration_type_3
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            configuration_type_4 = AzureKeyVaultConfigurationOutput.from_dict(data)

            return configuration_type_4

        configuration = _parse_configuration(d.pop("configuration"))

        secret_store_detail_response = cls(
            id=id,
            name=name,
            type_=type_,
            is_enabled=is_enabled,
            created_at=created_at,
            updated_at=updated_at,
            configuration=configuration,
        )

        return secret_store_detail_response



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define

from ..models.schema_76 import Schema76

T = TypeVar("T", bound="SecretStoreResponse")


@_attrs_define
class SecretStoreResponse:
    """
    Attributes:
        id (str):
        name (str):
        type_ (Schema76):
        is_enabled (bool):
        created_at (str):
        updated_at (str):
    """

    id: str
    name: str
    type_: Schema76
    is_enabled: bool
    created_at: str
    updated_at: str

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        type_ = self.type_.value

        is_enabled = self.is_enabled

        created_at = self.created_at

        updated_at = self.updated_at

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
                "type": type_,
                "isEnabled": is_enabled,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        type_ = Schema76(d.pop("type"))

        is_enabled = d.pop("isEnabled")

        created_at = d.pop("createdAt")

        updated_at = d.pop("updatedAt")

        secret_store_response = cls(
            id=id,
            name=name,
            type_=type_,
            is_enabled=is_enabled,
            created_at=created_at,
            updated_at=updated_at,
        )

        return secret_store_response



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define

T = TypeVar("T", bound="Schema57Type3")


@_attrs_define
class Schema57Type3:
    """
    Attributes:
        type_ (Literal['self']):
        id (str):
    """

    type_: Literal["self"]
    id: str

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        id = self.id

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "type": type_,
                "id": id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = cast(Literal["self"], d.pop("type"))
        if type_ != "self":
            raise ValueError(f"type must match const 'self', got '{type_}'")

        id = d.pop("id")

        schema_57_type_3 = cls(
            type_=type_,
            id=id,
        )

        return schema_57_type_3



from enum import Enum

class GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo(str, Enum):
    DETAILS = "details"
    PROFILE = "profile"
    SETTINGS = "settings"
    THANK_YOU_PAGE = "thank-you-page"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum

class McpAuditLogVerbosity(str, Enum):
    DETAILED = "detailed"
    DETAILED_WITH_ERROR = "detailed-with-error"
    OFF = "off"
    TERSE = "terse"
    VERBOSE = "verbose"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum

class PostSecretStoresBodyType(str, Enum):
    AWS_SECRETS_MANAGER = "aws_secrets_manager"
    AZURE_KEY_VAULT = "azure_key_vault"
    GCP_SECRET_MANAGER = "gcp_secret_manager"
    HASHICORP_VAULT = "hashicorp_vault"
    INFISICAL = "infisical"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="AuthMetadataTenantAbilities")


@_attrs_define
class AuthMetadataTenantAbilities:
    """
    Attributes:
        can_have_multiple_users (bool):
        can_use_sandbox (bool):
        can_use_hosted_servers (bool):
        can_use_local_servers (bool):
        can_update_seats (bool):
        max_enabled_servers (float):
        max_users (float):
        has_paid_features (bool):
    """

    can_have_multiple_users: bool
    can_use_sandbox: bool
    can_use_hosted_servers: bool
    can_use_local_servers: bool
    can_update_seats: bool
    max_enabled_servers: float
    max_users: float
    has_paid_features: bool

    def to_dict(self) -> dict[str, Any]:
        can_have_multiple_users = self.can_have_multiple_users

        can_use_sandbox = self.can_use_sandbox

        can_use_hosted_servers = self.can_use_hosted_servers

        can_use_local_servers = self.can_use_local_servers

        can_update_seats = self.can_update_seats

        max_enabled_servers = self.max_enabled_servers

        max_users = self.max_users

        has_paid_features = self.has_paid_features

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "canHaveMultipleUsers": can_have_multiple_users,
                "canUseSandbox": can_use_sandbox,
                "canUseHostedServers": can_use_hosted_servers,
                "canUseLocalServers": can_use_local_servers,
                "canUpdateSeats": can_update_seats,
                "maxEnabledServers": max_enabled_servers,
                "maxUsers": max_users,
                "hasPaidFeatures": has_paid_features,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        can_have_multiple_users = d.pop("canHaveMultipleUsers")

        can_use_sandbox = d.pop("canUseSandbox")

        can_use_hosted_servers = d.pop("canUseHostedServers")

        can_use_local_servers = d.pop("canUseLocalServers")

        can_update_seats = d.pop("canUpdateSeats")

        max_enabled_servers = d.pop("maxEnabledServers")

        max_users = d.pop("maxUsers")

        has_paid_features = d.pop("hasPaidFeatures")

        auth_metadata_tenant_abilities = cls(
            can_have_multiple_users=can_have_multiple_users,
            can_use_sandbox=can_use_sandbox,
            can_use_hosted_servers=can_use_hosted_servers,
            can_use_local_servers=can_use_local_servers,
            can_update_seats=can_update_seats,
            max_enabled_servers=max_enabled_servers,
            max_users=max_users,
            has_paid_features=has_paid_features,
        )

        return auth_metadata_tenant_abilities



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.schema_11 import Schema11
from ..types import UNSET, Unset
from typing import cast
from typing import Literal, cast






T = TypeVar("T", bound="PutMcpServersServerSlugConfigBodyType1")



@_attrs_define
class PutMcpServersServerSlugConfigBodyType1:
    """ 
        Attributes:
            type_ (Literal['hosted'] | Unset):
            runtime (Schema11 | Unset):
            env (list[list[str]] | Unset):
     """

    type_: Literal['hosted'] | Unset = UNSET
    runtime: Schema11 | Unset = UNSET
    env: list[list[str]] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        runtime: str | Unset = UNSET
        if not isinstance(self.runtime, Unset):
            runtime = self.runtime.value


        env: list[list[str]] | Unset = UNSET
        if not isinstance(self.env, Unset):
            env = []
            for componentsschemas_schema12_item_data in self.env:
                componentsschemas_schema12_item = []
                for componentsschemas_schema12_item_item_data in componentsschemas_schema12_item_data:
                    componentsschemas_schema12_item_item: str
                    componentsschemas_schema12_item_item = componentsschemas_schema12_item_item_data
                    componentsschemas_schema12_item.append(componentsschemas_schema12_item_item)


                env.append(componentsschemas_schema12_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if type_ is not UNSET:
            field_dict["type"] = type_
        if runtime is not UNSET:
            field_dict["runtime"] = runtime
        if env is not UNSET:
            field_dict["env"] = env

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = cast(Literal['hosted'] | Unset , d.pop("type", UNSET))
        if type_ != 'hosted'and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'hosted', got '{type_}'")

        _runtime = d.pop("runtime", UNSET)
        runtime: Schema11 | Unset
        if isinstance(_runtime,  Unset):
            runtime = UNSET
        else:
            runtime = Schema11(_runtime)




        _env = d.pop("env", UNSET)
        env: list[list[str]] | Unset = UNSET
        if _env is not UNSET:
            env = []
            for componentsschemas_schema12_item_data in _env:
                componentsschemas_schema12_item = []
                _componentsschemas_schema12_item = componentsschemas_schema12_item_data
                for componentsschemas_schema12_item_item_data in (_componentsschemas_schema12_item):
                    def _parse_componentsschemas_schema12_item_item(data: object) -> str:
                        return cast(str, data)

                    componentsschemas_schema12_item_item = _parse_componentsschemas_schema12_item_item(componentsschemas_schema12_item_item_data)

                    componentsschemas_schema12_item.append(componentsschemas_schema12_item_item)

                env.append(componentsschemas_schema12_item)


        put_mcp_servers_server_slug_config_body_type_1 = cls(
            type_=type_,
            runtime=runtime,
            env=env,
        )


        put_mcp_servers_server_slug_config_body_type_1.additional_properties = d
        return put_mcp_servers_server_slug_config_body_type_1

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.hosted_transport_config_output_runtime import HostedTransportConfigOutputRuntime
from ..types import UNSET, Unset
from typing import cast
from typing import Literal, cast






T = TypeVar("T", bound="HostedTransportConfigOutput")



@_attrs_define
class HostedTransportConfigOutput:
    """ 
        Attributes:
            type_ (Literal['hosted']):
            runtime (HostedTransportConfigOutputRuntime):
            env (list[list[str]] | Unset):
     """

    type_: Literal['hosted']
    runtime: HostedTransportConfigOutputRuntime
    env: list[list[str]] | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        runtime = self.runtime.value

        env: list[list[str]] | Unset = UNSET
        if not isinstance(self.env, Unset):
            env = []
            for env_item_data in self.env:
                env_item = []
                for env_item_item_data in env_item_data:
                    env_item_item: str
                    env_item_item = env_item_item_data
                    env_item.append(env_item_item)


                env.append(env_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "type": type_,
            "runtime": runtime,
        })
        if env is not UNSET:
            field_dict["env"] = env

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = cast(Literal['hosted'] , d.pop("type"))
        if type_ != 'hosted':
            raise ValueError(f"type must match const 'hosted', got '{type_}'")

        runtime = HostedTransportConfigOutputRuntime(d.pop("runtime"))




        _env = d.pop("env", UNSET)
        env: list[list[str]] | Unset = UNSET
        if _env is not UNSET:
            env = []
            for env_item_data in _env:
                env_item = []
                _env_item = env_item_data
                for env_item_item_data in (_env_item):
                    def _parse_env_item_item(data: object) -> str:
                        return cast(str, data)

                    env_item_item = _parse_env_item_item(env_item_item_data)

                    env_item.append(env_item_item)

                env.append(env_item)


        hosted_transport_config_output = cls(
            type_=type_,
            runtime=runtime,
            env=env,
        )

        return hosted_transport_config_output




from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.get_members_response_teams_item import GetMembersResponseTeamsItem
    from ..models.get_members_response_users_item import GetMembersResponseUsersItem


T = TypeVar("T", bound="GetMembersResponse")


@_attrs_define
class GetMembersResponse:
    """
    Attributes:
        teams (list[GetMembersResponseTeamsItem]):
        users (list[GetMembersResponseUsersItem]):
    """

    teams: list[GetMembersResponseTeamsItem]
    users: list[GetMembersResponseUsersItem]

    def to_dict(self) -> dict[str, Any]:
        teams = []
        for teams_item_data in self.teams:
            teams_item = teams_item_data.to_dict()
            teams.append(teams_item)

        users = []
        for users_item_data in self.users:
            users_item = users_item_data.to_dict()
            users.append(users_item)

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "teams": teams,
                "users": users,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_members_response_teams_item import GetMembersResponseTeamsItem
        from ..models.get_members_response_users_item import GetMembersResponseUsersItem

        d = dict(src_dict)
        teams = []
        _teams = d.pop("teams")
        for teams_item_data in _teams:
            teams_item = GetMembersResponseTeamsItem.from_dict(teams_item_data)

            teams.append(teams_item)

        users = []
        _users = d.pop("users")
        for users_item_data in _users:
            users_item = GetMembersResponseUsersItem.from_dict(users_item_data)

            users.append(users_item)

        get_members_response = cls(
            teams=teams,
            users=users,
        )

        return get_members_response



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define

from ..models.schema_69 import Schema69

T = TypeVar("T", bound="TeamMember")


@_attrs_define
class TeamMember:
    """
    Attributes:
        team_id (str):
        user_id (float):
        role (Schema69):
        created_at (str):
        updated_at (str):
    """

    team_id: str
    user_id: float
    role: Schema69
    created_at: str
    updated_at: str

    def to_dict(self) -> dict[str, Any]:
        team_id = self.team_id

        user_id = self.user_id

        role = self.role.value

        created_at = self.created_at

        updated_at = self.updated_at

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "teamId": team_id,
                "userId": user_id,
                "role": role,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        team_id = d.pop("teamId")

        user_id = d.pop("userId")

        role = Schema69(d.pop("role"))

        created_at = d.pop("createdAt")

        updated_at = d.pop("updatedAt")

        team_member = cls(
            team_id=team_id,
            user_id=user_id,
            role=role,
            created_at=created_at,
            updated_at=updated_at,
        )

        return team_member



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.server_credentials_dto import ServerCredentialsDto





T = TypeVar("T", bound="GetMcpServersServerSlugCredentialsUserResponse200")



@_attrs_define
class GetMcpServersServerSlugCredentialsUserResponse200:
    """ 
        Attributes:
            credentials (None | ServerCredentialsDto):
     """

    credentials: None | ServerCredentialsDto





    def to_dict(self) -> dict[str, Any]:
        from ..models.server_credentials_dto import ServerCredentialsDto
        credentials: dict[str, Any] | None
        if isinstance(self.credentials, ServerCredentialsDto):
            credentials = self.credentials.to_dict()
        else:
            credentials = self.credentials


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "credentials": credentials,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.server_credentials_dto import ServerCredentialsDto
        d = dict(src_dict)
        def _parse_credentials(data: object) -> None | ServerCredentialsDto:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                credentials_type_0 = ServerCredentialsDto.from_dict(data)



                return credentials_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ServerCredentialsDto, data)

        credentials = _parse_credentials(d.pop("credentials"))


        get_mcp_servers_server_slug_credentials_user_response_200 = cls(
            credentials=credentials,
        )

        return get_mcp_servers_server_slug_credentials_user_response_200




from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.sandbox_dto import SandboxDto


T = TypeVar("T", bound="CreateSandboxResponse")


@_attrs_define
class CreateSandboxResponse:
    """
    Attributes:
        sandbox (SandboxDto):
    """

    sandbox: SandboxDto

    def to_dict(self) -> dict[str, Any]:
        sandbox = self.sandbox.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "sandbox": sandbox,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.sandbox_dto import SandboxDto

        d = dict(src_dict)
        sandbox = SandboxDto.from_dict(d.pop("sandbox"))

        create_sandbox_response = cls(
            sandbox=sandbox,
        )

        return create_sandbox_response



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.schema_0 import Schema0






T = TypeVar("T", bound="UpdateUserRequest")



@_attrs_define
class UpdateUserRequest:
    """ 
        Attributes:
            email (str):
            name (str):
            role (Schema0):
            is_disabled (bool):
     """

    email: str
    name: str
    role: Schema0
    is_disabled: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        email = self.email

        name = self.name

        role = self.role.value

        is_disabled = self.is_disabled


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "email": email,
            "name": name,
            "role": role,
            "isDisabled": is_disabled,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        email = d.pop("email")

        name = d.pop("name")

        role = Schema0(d.pop("role"))




        is_disabled = d.pop("isDisabled")

        update_user_request = cls(
            email=email,
            name=name,
            role=role,
            is_disabled=is_disabled,
        )


        update_user_request.additional_properties = d
        return update_user_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.server import Server
    from ..models.server_member_type_0 import ServerMemberType0
    from ..models.server_member_type_1 import ServerMemberType1


T = TypeVar("T", bound="GetTeamsTeamIdServersResponse200PermissionsItem")


@_attrs_define
class GetTeamsTeamIdServersResponse200PermissionsItem:
    """
    Attributes:
        permission (ServerMemberType0 | ServerMemberType1):
        server (Server):
    """

    permission: ServerMemberType0 | ServerMemberType1
    server: Server

    def to_dict(self) -> dict[str, Any]:
        from ..models.server_member_type_0 import ServerMemberType0

        permission: dict[str, Any]
        if isinstance(self.permission, ServerMemberType0):
            permission = self.permission.to_dict()
        else:
            permission = self.permission.to_dict()

        server = self.server.to_dict()

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "permission": permission,
                "server": server,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.server import Server
        from ..models.server_member_type_0 import ServerMemberType0
        from ..models.server_member_type_1 import ServerMemberType1

        d = dict(src_dict)

        def _parse_permission(data: object) -> ServerMemberType0 | ServerMemberType1:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_server_member_type_0 = ServerMemberType0.from_dict(data)

                return componentsschemas_server_member_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_server_member_type_1 = ServerMemberType1.from_dict(data)

            return componentsschemas_server_member_type_1

        permission = _parse_permission(d.pop("permission"))

        server = Server.from_dict(d.pop("server"))

        get_teams_team_id_servers_response_200_permissions_item = cls(
            permission=permission,
            server=server,
        )

        return get_teams_team_id_servers_response_200_permissions_item



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.server_o_auth_metadata import ServerOAuthMetadata





T = TypeVar("T", bound="GetMcpServersServerSlugOauthDiscoverResponse200")



@_attrs_define
class GetMcpServersServerSlugOauthDiscoverResponse200:
    """ 
        Attributes:
            oauth_metadata (None | ServerOAuthMetadata):
     """

    oauth_metadata: None | ServerOAuthMetadata





    def to_dict(self) -> dict[str, Any]:
        from ..models.server_o_auth_metadata import ServerOAuthMetadata
        oauth_metadata: dict[str, Any] | None
        if isinstance(self.oauth_metadata, ServerOAuthMetadata):
            oauth_metadata = self.oauth_metadata.to_dict()
        else:
            oauth_metadata = self.oauth_metadata


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "oauthMetadata": oauth_metadata,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.server_o_auth_metadata import ServerOAuthMetadata
        d = dict(src_dict)
        def _parse_oauth_metadata(data: object) -> None | ServerOAuthMetadata:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                oauth_metadata_type_0 = ServerOAuthMetadata.from_dict(data)



                return oauth_metadata_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ServerOAuthMetadata, data)

        oauth_metadata = _parse_oauth_metadata(d.pop("oauthMetadata"))


        get_mcp_servers_server_slug_oauth_discover_response_200 = cls(
            oauth_metadata=oauth_metadata,
        )

        return get_mcp_servers_server_slug_oauth_discover_response_200




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast






T = TypeVar("T", bound="PersonalAccessToken")



@_attrs_define
class PersonalAccessToken:
    """ 
        Attributes:
            tenant_id (str):
            user_id (float):
            id (str):
            name (str):
            api_key (str):
            profile_id (None | str):
            created_at (str):
            last_used_at (None | str):
     """

    tenant_id: str
    user_id: float
    id: str
    name: str
    api_key: str
    profile_id: None | str
    created_at: str
    last_used_at: None | str





    def to_dict(self) -> dict[str, Any]:
        tenant_id = self.tenant_id

        user_id = self.user_id

        id = self.id

        name = self.name

        api_key = self.api_key

        profile_id: None | str
        profile_id = self.profile_id

        created_at = self.created_at

        last_used_at: None | str
        last_used_at = self.last_used_at


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "tenantId": tenant_id,
            "userId": user_id,
            "id": id,
            "name": name,
            "apiKey": api_key,
            "profileId": profile_id,
            "createdAt": created_at,
            "lastUsedAt": last_used_at,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        tenant_id = d.pop("tenantId")

        user_id = d.pop("userId")

        id = d.pop("id")

        name = d.pop("name")

        api_key = d.pop("apiKey")

        def _parse_profile_id(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        profile_id = _parse_profile_id(d.pop("profileId"))


        created_at = d.pop("createdAt")

        def _parse_last_used_at(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        last_used_at = _parse_last_used_at(d.pop("lastUsedAt"))


        personal_access_token = cls(
            tenant_id=tenant_id,
            user_id=user_id,
            id=id,
            name=name,
            api_key=api_key,
            profile_id=profile_id,
            created_at=created_at,
            last_used_at=last_used_at,
        )

        return personal_access_token




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="SandboxWithStatusDtoStatus")


@_attrs_define
class SandboxWithStatusDtoStatus:
    """
    Attributes:
        is_deployed (bool):
        is_available (bool):
        is_stabilizing (bool):
    """

    is_deployed: bool
    is_available: bool
    is_stabilizing: bool

    def to_dict(self) -> dict[str, Any]:
        is_deployed = self.is_deployed

        is_available = self.is_available

        is_stabilizing = self.is_stabilizing

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "isDeployed": is_deployed,
                "isAvailable": is_available,
                "isStabilizing": is_stabilizing,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        is_deployed = d.pop("isDeployed")

        is_available = d.pop("isAvailable")

        is_stabilizing = d.pop("isStabilizing")

        sandbox_with_status_dto_status = cls(
            is_deployed=is_deployed,
            is_available=is_available,
            is_stabilizing=is_stabilizing,
        )

        return sandbox_with_status_dto_status



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.get_subscription_response_subscription_type_0_card_type_0 import GetSubscriptionResponseSubscriptionType0CardType0





T = TypeVar("T", bound="GetSubscriptionResponseSubscriptionType0")



@_attrs_define
class GetSubscriptionResponseSubscriptionType0:
    """ 
        Attributes:
            has_payment_method (bool):
            trial_ends_at (None | str):
            status (str):
            current_period_end (str):
            cancels_at (None | str):
            per_seat_amount (float):
            currency (str):
            card (GetSubscriptionResponseSubscriptionType0CardType0 | None):
            payment_type (None | str | Unset):
     """

    has_payment_method: bool
    trial_ends_at: None | str
    status: str
    current_period_end: str
    cancels_at: None | str
    per_seat_amount: float
    currency: str
    card: GetSubscriptionResponseSubscriptionType0CardType0 | None
    payment_type: None | str | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.get_subscription_response_subscription_type_0_card_type_0 import GetSubscriptionResponseSubscriptionType0CardType0
        has_payment_method = self.has_payment_method

        trial_ends_at: None | str
        trial_ends_at = self.trial_ends_at

        status = self.status

        current_period_end = self.current_period_end

        cancels_at: None | str
        cancels_at = self.cancels_at

        per_seat_amount = self.per_seat_amount

        currency = self.currency

        card: dict[str, Any] | None
        if isinstance(self.card, GetSubscriptionResponseSubscriptionType0CardType0):
            card = self.card.to_dict()
        else:
            card = self.card

        payment_type: None | str | Unset
        if isinstance(self.payment_type, Unset):
            payment_type = UNSET
        else:
            payment_type = self.payment_type


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "hasPaymentMethod": has_payment_method,
            "trialEndsAt": trial_ends_at,
            "status": status,
            "currentPeriodEnd": current_period_end,
            "cancelsAt": cancels_at,
            "perSeatAmount": per_seat_amount,
            "currency": currency,
            "card": card,
        })
        if payment_type is not UNSET:
            field_dict["paymentType"] = payment_type

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_subscription_response_subscription_type_0_card_type_0 import GetSubscriptionResponseSubscriptionType0CardType0
        d = dict(src_dict)
        has_payment_method = d.pop("hasPaymentMethod")

        def _parse_trial_ends_at(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        trial_ends_at = _parse_trial_ends_at(d.pop("trialEndsAt"))


        status = d.pop("status")

        current_period_end = d.pop("currentPeriodEnd")

        def _parse_cancels_at(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        cancels_at = _parse_cancels_at(d.pop("cancelsAt"))


        per_seat_amount = d.pop("perSeatAmount")

        currency = d.pop("currency")

        def _parse_card(data: object) -> GetSubscriptionResponseSubscriptionType0CardType0 | None:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                card_type_0 = GetSubscriptionResponseSubscriptionType0CardType0.from_dict(data)



                return card_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(GetSubscriptionResponseSubscriptionType0CardType0 | None, data)

        card = _parse_card(d.pop("card"))


        def _parse_payment_type(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        payment_type = _parse_payment_type(d.pop("paymentType", UNSET))


        get_subscription_response_subscription_type_0 = cls(
            has_payment_method=has_payment_method,
            trial_ends_at=trial_ends_at,
            status=status,
            current_period_end=current_period_end,
            cancels_at=cancels_at,
            per_seat_amount=per_seat_amount,
            currency=currency,
            card=card,
            payment_type=payment_type,
        )

        return get_subscription_response_subscription_type_0




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.team_invitation import TeamInvitation





T = TypeVar("T", bound="GetTeamsTeamIdInvitationsResponse200")



@_attrs_define
class GetTeamsTeamIdInvitationsResponse200:
    """ 
        Attributes:
            invitations (list[TeamInvitation]):
     """

    invitations: list[TeamInvitation]





    def to_dict(self) -> dict[str, Any]:
        from ..models.team_invitation import TeamInvitation
        invitations = []
        for invitations_item_data in self.invitations:
            invitations_item = invitations_item_data.to_dict()
            invitations.append(invitations_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "invitations": invitations,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.team_invitation import TeamInvitation
        d = dict(src_dict)
        invitations = []
        _invitations = d.pop("invitations")
        for invitations_item_data in (_invitations):
            invitations_item = TeamInvitation.from_dict(invitations_item_data)



            invitations.append(invitations_item)


        get_teams_team_id_invitations_response_200 = cls(
            invitations=invitations,
        )

        return get_teams_team_id_invitations_response_200




from enum import Enum

class UserIdentityType(str, Enum):
    EXTERNAL_OIDC = "external-oidc"
    EXTERNAL_SAML = "external-saml"
    NATIVE = "native"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any
from urllib.parse import quote

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.post_sandboxes_sandbox_id_write_file_response_200 import PostSandboxesSandboxIdWriteFileResponse200
from ...types import UNSET, Response, Unset


def _get_kwargs(
    sandbox_id: str,
    *,
    body: Any | Unset = UNSET,
    path: str,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    params["path"] = path

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/sandboxes/{sandbox_id}/write-file".format(
            sandbox_id=quote(str(sandbox_id), safe=""),
        ),
        "params": params,
    }

    if not isinstance(body, Unset):
        _kwargs["content"] = body.payload

    headers["Content-Type"] = "application/octet-stream"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> PostSandboxesSandboxIdWriteFileResponse200 | None:
    if response.status_code == 200:
        response_200 = PostSandboxesSandboxIdWriteFileResponse200.from_dict(response.json())

        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[PostSandboxesSandboxIdWriteFileResponse200]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    sandbox_id: str,
    *,
    client: AuthenticatedClient | Client,
    body: Any | Unset = UNSET,
    path: str,
) -> Response[PostSandboxesSandboxIdWriteFileResponse200]:
    """
    Args:
        sandbox_id (str):
        path (str):
        body (Any | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PostSandboxesSandboxIdWriteFileResponse200]
    """

    kwargs = _get_kwargs(
        sandbox_id=sandbox_id,
        body=body,
        path=path,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    sandbox_id: str,
    *,
    client: AuthenticatedClient | Client,
    body: Any | Unset = UNSET,
    path: str,
) -> PostSandboxesSandboxIdWriteFileResponse200 | None:
    """
    Args:
        sandbox_id (str):
        path (str):
        body (Any | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PostSandboxesSandboxIdWriteFileResponse200
    """

    return sync_detailed(
        sandbox_id=sandbox_id,
        client=client,
        body=body,
        path=path,
    ).parsed


async def asyncio_detailed(
    sandbox_id: str,
    *,
    client: AuthenticatedClient | Client,
    body: Any | Unset = UNSET,
    path: str,
) -> Response[PostSandboxesSandboxIdWriteFileResponse200]:
    """
    Args:
        sandbox_id (str):
        path (str):
        body (Any | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PostSandboxesSandboxIdWriteFileResponse200]
    """

    kwargs = _get_kwargs(
        sandbox_id=sandbox_id,
        body=body,
        path=path,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    sandbox_id: str,
    *,
    client: AuthenticatedClient | Client,
    body: Any | Unset = UNSET,
    path: str,
) -> PostSandboxesSandboxIdWriteFileResponse200 | None:
    """
    Args:
        sandbox_id (str):
        path (str):
        body (Any | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PostSandboxesSandboxIdWriteFileResponse200
    """

    return (
        await asyncio_detailed(
            sandbox_id=sandbox_id,
            client=client,
            body=body,
            path=path,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.post_mcp_servers_server_slug_credentials_copy_response_200 import PostMcpServersServerSlugCredentialsCopyResponse200
from ...models.schema_32 import Schema32
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    server_slug: str,
    *,
    id: str,
    to: Schema32,
    profile_id: str | Unset = UNSET,
    user_sub: str | Unset = UNSET,

) -> dict[str, Any]:
    

    

    params: dict[str, Any] = {}

    params["id"] = id

    json_to = to.value
    params["to"] = json_to

    params["profileId"] = profile_id

    params["userSub"] = user_sub


    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}


    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/mcp-servers/{server_slug}/credentials/copy".format(server_slug=quote(str(server_slug), safe=""),),
        "params": params,
    }


    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> PostMcpServersServerSlugCredentialsCopyResponse200 | None:
    if response.status_code == 200:
        response_200 = PostMcpServersServerSlugCredentialsCopyResponse200.from_dict(response.json())



        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[PostMcpServersServerSlugCredentialsCopyResponse200]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    server_slug: str,
    *,
    client: AuthenticatedClient | Client,
    id: str,
    to: Schema32,
    profile_id: str | Unset = UNSET,
    user_sub: str | Unset = UNSET,

) -> Response[PostMcpServersServerSlugCredentialsCopyResponse200]:
    """  Copy credentials to the another scope

    Args:
        server_slug (str):
        id (str): ID of the credentials to copy
        to (Schema32):
        profile_id (str | Unset): Target profile ID when copying to profile scope
        user_sub (str | Unset): Target user sub when copying to user scope which is not the
            currently authenticated user

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PostMcpServersServerSlugCredentialsCopyResponse200]
     """


    kwargs = _get_kwargs(
        server_slug=server_slug,
id=id,
to=to,
profile_id=profile_id,
user_sub=user_sub,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    server_slug: str,
    *,
    client: AuthenticatedClient | Client,
    id: str,
    to: Schema32,
    profile_id: str | Unset = UNSET,
    user_sub: str | Unset = UNSET,

) -> PostMcpServersServerSlugCredentialsCopyResponse200 | None:
    """  Copy credentials to the another scope

    Args:
        server_slug (str):
        id (str): ID of the credentials to copy
        to (Schema32):
        profile_id (str | Unset): Target profile ID when copying to profile scope
        user_sub (str | Unset): Target user sub when copying to user scope which is not the
            currently authenticated user

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PostMcpServersServerSlugCredentialsCopyResponse200
     """


    return sync_detailed(
        server_slug=server_slug,
client=client,
id=id,
to=to,
profile_id=profile_id,
user_sub=user_sub,

    ).parsed

async def asyncio_detailed(
    server_slug: str,
    *,
    client: AuthenticatedClient | Client,
    id: str,
    to: Schema32,
    profile_id: str | Unset = UNSET,
    user_sub: str | Unset = UNSET,

) -> Response[PostMcpServersServerSlugCredentialsCopyResponse200]:
    """  Copy credentials to the another scope

    Args:
        server_slug (str):
        id (str): ID of the credentials to copy
        to (Schema32):
        profile_id (str | Unset): Target profile ID when copying to profile scope
        user_sub (str | Unset): Target user sub when copying to user scope which is not the
            currently authenticated user

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PostMcpServersServerSlugCredentialsCopyResponse200]
     """


    kwargs = _get_kwargs(
        server_slug=server_slug,
id=id,
to=to,
profile_id=profile_id,
user_sub=user_sub,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    server_slug: str,
    *,
    client: AuthenticatedClient | Client,
    id: str,
    to: Schema32,
    profile_id: str | Unset = UNSET,
    user_sub: str | Unset = UNSET,

) -> PostMcpServersServerSlugCredentialsCopyResponse200 | None:
    """  Copy credentials to the another scope

    Args:
        server_slug (str):
        id (str): ID of the credentials to copy
        to (Schema32):
        profile_id (str | Unset): Target profile ID when copying to profile scope
        user_sub (str | Unset): Target user sub when copying to user scope which is not the
            currently authenticated user

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PostMcpServersServerSlugCredentialsCopyResponse200
     """


    return (await asyncio_detailed(
        server_slug=server_slug,
client=client,
id=id,
to=to,
profile_id=profile_id,
user_sub=user_sub,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.get_teams_team_id_servers_response_200_permissions_item import GetTeamsTeamIdServersResponse200PermissionsItem





T = TypeVar("T", bound="GetTeamsTeamIdServersResponse200")



@_attrs_define
class GetTeamsTeamIdServersResponse200:
    """ 
        Attributes:
            permissions (list[GetTeamsTeamIdServersResponse200PermissionsItem]):
     """

    permissions: list[GetTeamsTeamIdServersResponse200PermissionsItem]





    def to_dict(self) -> dict[str, Any]:
        from ..models.get_teams_team_id_servers_response_200_permissions_item import GetTeamsTeamIdServersResponse200PermissionsItem
        permissions = []
        for permissions_item_data in self.permissions:
            permissions_item = permissions_item_data.to_dict()
            permissions.append(permissions_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "permissions": permissions,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_teams_team_id_servers_response_200_permissions_item import GetTeamsTeamIdServersResponse200PermissionsItem
        d = dict(src_dict)
        permissions = []
        _permissions = d.pop("permissions")
        for permissions_item_data in (_permissions):
            permissions_item = GetTeamsTeamIdServersResponse200PermissionsItem.from_dict(permissions_item_data)



            permissions.append(permissions_item)


        get_teams_team_id_servers_response_200 = cls(
            permissions=permissions,
        )

        return get_teams_team_id_servers_response_200




from enum import Enum

class CreateServerRequestTransportType(str, Enum):
    HOSTED = "hosted"
    HTTPSTREAMING = "httpstreaming"
    SSE = "sse"
    STDIO = "stdio"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="ServerOAuthMetadataAs")



@_attrs_define
class ServerOAuthMetadataAs:
    """ 
        Attributes:
            issuer (str):  Default: ''.
            authorize_endpoint (str):  Default: ''.
            device_authorization_endpoint (str):  Default: ''.
            token_endpoint (str):  Default: ''.
            extra_authorization_parameters (str):  Default: ''.
            supports_pkce (bool):  Default: False.
            supported_client_auth_methods (list[str]):
            supports_dynamic_client_registration (bool):  Default: False.
            service_documentation (str):  Default: ''.
            metadata_url (str):  Default: ''.
            available_scopes (list[str]):
            registration_endpoint (str | Unset):
            no_dynamic_client_registration_reason (None | str | Unset):
     """

    supported_client_auth_methods: list[str]
    available_scopes: list[str]
    issuer: str = ''
    authorize_endpoint: str = ''
    device_authorization_endpoint: str = ''
    token_endpoint: str = ''
    extra_authorization_parameters: str = ''
    supports_pkce: bool = False
    supports_dynamic_client_registration: bool = False
    service_documentation: str = ''
    metadata_url: str = ''
    registration_endpoint: str | Unset = UNSET
    no_dynamic_client_registration_reason: None | str | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        issuer = self.issuer

        authorize_endpoint = self.authorize_endpoint

        device_authorization_endpoint = self.device_authorization_endpoint

        token_endpoint = self.token_endpoint

        extra_authorization_parameters = self.extra_authorization_parameters

        supports_pkce = self.supports_pkce

        supported_client_auth_methods = self.supported_client_auth_methods



        supports_dynamic_client_registration = self.supports_dynamic_client_registration

        service_documentation = self.service_documentation

        metadata_url = self.metadata_url

        available_scopes = self.available_scopes



        registration_endpoint = self.registration_endpoint

        no_dynamic_client_registration_reason: None | str | Unset
        if isinstance(self.no_dynamic_client_registration_reason, Unset):
            no_dynamic_client_registration_reason = UNSET
        else:
            no_dynamic_client_registration_reason = self.no_dynamic_client_registration_reason


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "issuer": issuer,
            "authorizeEndpoint": authorize_endpoint,
            "deviceAuthorizationEndpoint": device_authorization_endpoint,
            "tokenEndpoint": token_endpoint,
            "extraAuthorizationParameters": extra_authorization_parameters,
            "supportsPKCE": supports_pkce,
            "supportedClientAuthMethods": supported_client_auth_methods,
            "supportsDynamicClientRegistration": supports_dynamic_client_registration,
            "serviceDocumentation": service_documentation,
            "metadataUrl": metadata_url,
            "availableScopes": available_scopes,
        })
        if registration_endpoint is not UNSET:
            field_dict["registrationEndpoint"] = registration_endpoint
        if no_dynamic_client_registration_reason is not UNSET:
            field_dict["noDynamicClientRegistrationReason"] = no_dynamic_client_registration_reason

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        issuer = d.pop("issuer")

        authorize_endpoint = d.pop("authorizeEndpoint")

        device_authorization_endpoint = d.pop("deviceAuthorizationEndpoint")

        token_endpoint = d.pop("tokenEndpoint")

        extra_authorization_parameters = d.pop("extraAuthorizationParameters")

        supports_pkce = d.pop("supportsPKCE")

        supported_client_auth_methods = cast(list[str], d.pop("supportedClientAuthMethods"))


        supports_dynamic_client_registration = d.pop("supportsDynamicClientRegistration")

        service_documentation = d.pop("serviceDocumentation")

        metadata_url = d.pop("metadataUrl")

        available_scopes = cast(list[str], d.pop("availableScopes"))


        registration_endpoint = d.pop("registrationEndpoint", UNSET)

        def _parse_no_dynamic_client_registration_reason(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        no_dynamic_client_registration_reason = _parse_no_dynamic_client_registration_reason(d.pop("noDynamicClientRegistrationReason", UNSET))


        server_o_auth_metadata_as = cls(
            issuer=issuer,
            authorize_endpoint=authorize_endpoint,
            device_authorization_endpoint=device_authorization_endpoint,
            token_endpoint=token_endpoint,
            extra_authorization_parameters=extra_authorization_parameters,
            supports_pkce=supports_pkce,
            supported_client_auth_methods=supported_client_auth_methods,
            supports_dynamic_client_registration=supports_dynamic_client_registration,
            service_documentation=service_documentation,
            metadata_url=metadata_url,
            available_scopes=available_scopes,
            registration_endpoint=registration_endpoint,
            no_dynamic_client_registration_reason=no_dynamic_client_registration_reason,
        )

        return server_o_auth_metadata_as




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.tenant_oidc_configuration_client_auth_method import TenantOidcConfigurationClientAuthMethod






T = TypeVar("T", bound="TenantOidcConfiguration")



@_attrs_define
class TenantOidcConfiguration:
    """ 
        Attributes:
            is_enabled (bool):
            display_name (str):
            issuer (str):
            authorize_endpoint (str):
            extra_parameters (str):
            token_endpoint (str):
            user_info_endpoint (str):
            introspection_endpoint (str):
            jwks_uri (str):
            client_id (str):
            client_secret (str):
            client_auth_method (TenantOidcConfigurationClientAuthMethod):
            scopes (str):
     """

    is_enabled: bool
    display_name: str
    issuer: str
    authorize_endpoint: str
    extra_parameters: str
    token_endpoint: str
    user_info_endpoint: str
    introspection_endpoint: str
    jwks_uri: str
    client_id: str
    client_secret: str
    client_auth_method: TenantOidcConfigurationClientAuthMethod
    scopes: str





    def to_dict(self) -> dict[str, Any]:
        is_enabled = self.is_enabled

        display_name = self.display_name

        issuer = self.issuer

        authorize_endpoint = self.authorize_endpoint

        extra_parameters = self.extra_parameters

        token_endpoint = self.token_endpoint

        user_info_endpoint = self.user_info_endpoint

        introspection_endpoint = self.introspection_endpoint

        jwks_uri = self.jwks_uri

        client_id = self.client_id

        client_secret = self.client_secret

        client_auth_method = self.client_auth_method.value

        scopes = self.scopes


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "isEnabled": is_enabled,
            "displayName": display_name,
            "issuer": issuer,
            "authorizeEndpoint": authorize_endpoint,
            "extraParameters": extra_parameters,
            "tokenEndpoint": token_endpoint,
            "userInfoEndpoint": user_info_endpoint,
            "introspectionEndpoint": introspection_endpoint,
            "jwksUri": jwks_uri,
            "clientId": client_id,
            "clientSecret": client_secret,
            "clientAuthMethod": client_auth_method,
            "scopes": scopes,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        is_enabled = d.pop("isEnabled")

        display_name = d.pop("displayName")

        issuer = d.pop("issuer")

        authorize_endpoint = d.pop("authorizeEndpoint")

        extra_parameters = d.pop("extraParameters")

        token_endpoint = d.pop("tokenEndpoint")

        user_info_endpoint = d.pop("userInfoEndpoint")

        introspection_endpoint = d.pop("introspectionEndpoint")

        jwks_uri = d.pop("jwksUri")

        client_id = d.pop("clientId")

        client_secret = d.pop("clientSecret")

        client_auth_method = TenantOidcConfigurationClientAuthMethod(d.pop("clientAuthMethod"))




        scopes = d.pop("scopes")

        tenant_oidc_configuration = cls(
            is_enabled=is_enabled,
            display_name=display_name,
            issuer=issuer,
            authorize_endpoint=authorize_endpoint,
            extra_parameters=extra_parameters,
            token_endpoint=token_endpoint,
            user_info_endpoint=user_info_endpoint,
            introspection_endpoint=introspection_endpoint,
            jwks_uri=jwks_uri,
            client_id=client_id,
            client_secret=client_secret,
            client_auth_method=client_auth_method,
            scopes=scopes,
        )

        return tenant_oidc_configuration




from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.hosted_transport_config import HostedTransportConfig
    from ..models.http_streaming_transport_config import HttpStreamingTransportConfig
    from ..models.server_authorization import ServerAuthorization
    from ..models.sse_transport_config import SseTransportConfig
    from ..models.stdio_transport_config import StdioTransportConfig
    from ..models.update_server_request_oauth_client_configuration_type_0 import (
        UpdateServerRequestOauthClientConfigurationType0,
    )
    from ..models.update_server_request_oauth_metadata_type_0 import UpdateServerRequestOauthMetadataType0
    from ..models.update_server_request_transport_config_type_4 import UpdateServerRequestTransportConfigType4


T = TypeVar("T", bound="UpdateServerRequest")


@_attrs_define
class UpdateServerRequest:
    """
    Attributes:
        slug (str | Unset):
        description (str | Unset):
        logo_url (str | Unset):
        url (str | Unset): The URL of the remote MCP server
        authorization (ServerAuthorization | Unset):
        transport_config (HostedTransportConfig | HttpStreamingTransportConfig | SseTransportConfig |
            StdioTransportConfig | Unset | UpdateServerRequestTransportConfigType4):
        oauth_metadata (None | Unset | UpdateServerRequestOauthMetadataType0):
        oauth_client_configuration (None | Unset | UpdateServerRequestOauthClientConfigurationType0):
    """

    slug: str | Unset = UNSET
    description: str | Unset = UNSET
    logo_url: str | Unset = UNSET
    url: str | Unset = UNSET
    authorization: ServerAuthorization | Unset = UNSET
    transport_config: (
        HostedTransportConfig
        | HttpStreamingTransportConfig
        | SseTransportConfig
        | StdioTransportConfig
        | Unset
        | UpdateServerRequestTransportConfigType4
    ) = UNSET
    oauth_metadata: None | Unset | UpdateServerRequestOauthMetadataType0 = UNSET
    oauth_client_configuration: None | Unset | UpdateServerRequestOauthClientConfigurationType0 = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.hosted_transport_config import HostedTransportConfig
        from ..models.http_streaming_transport_config import HttpStreamingTransportConfig
        from ..models.sse_transport_config import SseTransportConfig
        from ..models.stdio_transport_config import StdioTransportConfig
        from ..models.update_server_request_oauth_client_configuration_type_0 import (
            UpdateServerRequestOauthClientConfigurationType0,
        )
        from ..models.update_server_request_oauth_metadata_type_0 import UpdateServerRequestOauthMetadataType0

        slug = self.slug

        description = self.description

        logo_url = self.logo_url

        url = self.url

        authorization: dict[str, Any] | Unset = UNSET
        if not isinstance(self.authorization, Unset):
            authorization = self.authorization.to_dict()

        transport_config: dict[str, Any] | Unset
        if isinstance(self.transport_config, Unset):
            transport_config = UNSET
        elif isinstance(self.transport_config, HttpStreamingTransportConfig):
            transport_config = self.transport_config.to_dict()
        elif isinstance(self.transport_config, HostedTransportConfig):
            transport_config = self.transport_config.to_dict()
        elif isinstance(self.transport_config, StdioTransportConfig):
            transport_config = self.transport_config.to_dict()
        elif isinstance(self.transport_config, SseTransportConfig):
            transport_config = self.transport_config.to_dict()
        else:
            transport_config = self.transport_config.to_dict()

        oauth_metadata: dict[str, Any] | None | Unset
        if isinstance(self.oauth_metadata, Unset):
            oauth_metadata = UNSET
        elif isinstance(self.oauth_metadata, UpdateServerRequestOauthMetadataType0):
            oauth_metadata = self.oauth_metadata.to_dict()
        else:
            oauth_metadata = self.oauth_metadata

        oauth_client_configuration: dict[str, Any] | None | Unset
        if isinstance(self.oauth_client_configuration, Unset):
            oauth_client_configuration = UNSET
        elif isinstance(self.oauth_client_configuration, UpdateServerRequestOauthClientConfigurationType0):
            oauth_client_configuration = self.oauth_client_configuration.to_dict()
        else:
            oauth_client_configuration = self.oauth_client_configuration

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if slug is not UNSET:
            field_dict["slug"] = slug
        if description is not UNSET:
            field_dict["description"] = description
        if logo_url is not UNSET:
            field_dict["logoUrl"] = logo_url
        if url is not UNSET:
            field_dict["url"] = url
        if authorization is not UNSET:
            field_dict["authorization"] = authorization
        if transport_config is not UNSET:
            field_dict["transportConfig"] = transport_config
        if oauth_metadata is not UNSET:
            field_dict["oauthMetadata"] = oauth_metadata
        if oauth_client_configuration is not UNSET:
            field_dict["oauthClientConfiguration"] = oauth_client_configuration

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.hosted_transport_config import HostedTransportConfig
        from ..models.http_streaming_transport_config import HttpStreamingTransportConfig
        from ..models.server_authorization import ServerAuthorization
        from ..models.sse_transport_config import SseTransportConfig
        from ..models.stdio_transport_config import StdioTransportConfig
        from ..models.update_server_request_oauth_client_configuration_type_0 import (
            UpdateServerRequestOauthClientConfigurationType0,
        )
        from ..models.update_server_request_oauth_metadata_type_0 import UpdateServerRequestOauthMetadataType0
        from ..models.update_server_request_transport_config_type_4 import UpdateServerRequestTransportConfigType4

        d = dict(src_dict)
        slug = d.pop("slug", UNSET)

        description = d.pop("description", UNSET)

        logo_url = d.pop("logoUrl", UNSET)

        url = d.pop("url", UNSET)

        _authorization = d.pop("authorization", UNSET)
        authorization: ServerAuthorization | Unset
        if isinstance(_authorization, Unset):
            authorization = UNSET
        else:
            authorization = ServerAuthorization.from_dict(_authorization)

        def _parse_transport_config(
            data: object,
        ) -> (
            HostedTransportConfig
            | HttpStreamingTransportConfig
            | SseTransportConfig
            | StdioTransportConfig
            | Unset
            | UpdateServerRequestTransportConfigType4
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                transport_config_type_0 = HttpStreamingTransportConfig.from_dict(data)

                return transport_config_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                transport_config_type_1 = HostedTransportConfig.from_dict(data)

                return transport_config_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                transport_config_type_2 = StdioTransportConfig.from_dict(data)

                return transport_config_type_2
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                transport_config_type_3 = SseTransportConfig.from_dict(data)

                return transport_config_type_3
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            transport_config_type_4 = UpdateServerRequestTransportConfigType4.from_dict(data)

            return transport_config_type_4

        transport_config = _parse_transport_config(d.pop("transportConfig", UNSET))

        def _parse_oauth_metadata(data: object) -> None | Unset | UpdateServerRequestOauthMetadataType0:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                oauth_metadata_type_0 = UpdateServerRequestOauthMetadataType0.from_dict(data)

                return oauth_metadata_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UpdateServerRequestOauthMetadataType0, data)

        oauth_metadata = _parse_oauth_metadata(d.pop("oauthMetadata", UNSET))

        def _parse_oauth_client_configuration(
            data: object,
        ) -> None | Unset | UpdateServerRequestOauthClientConfigurationType0:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                oauth_client_configuration_type_0 = UpdateServerRequestOauthClientConfigurationType0.from_dict(data)

                return oauth_client_configuration_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UpdateServerRequestOauthClientConfigurationType0, data)

        oauth_client_configuration = _parse_oauth_client_configuration(d.pop("oauthClientConfiguration", UNSET))

        update_server_request = cls(
            slug=slug,
            description=description,
            logo_url=logo_url,
            url=url,
            authorization=authorization,
            transport_config=transport_config,
            oauth_metadata=oauth_metadata,
            oauth_client_configuration=oauth_client_configuration,
        )

        update_server_request.additional_properties = d
        return update_server_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.hosted_transport_config_output import HostedTransportConfigOutput
    from ..models.http_streaming_transport_config_output import HttpStreamingTransportConfigOutput
    from ..models.schema_57_type_3 import Schema57Type3
    from ..models.server_authorization_output import ServerAuthorizationOutput
    from ..models.server_o_auth_client_configuration import ServerOAuthClientConfiguration
    from ..models.server_o_auth_metadata import ServerOAuthMetadata
    from ..models.sse_transport_config_output import SseTransportConfigOutput
    from ..models.stdio_transport_config_output import StdioTransportConfigOutput


T = TypeVar("T", bound="ServerDto")


@_attrs_define
class ServerDto:
    """
    Attributes:
        id (float):
        slug (str):
        tenant_id (str):
        name (str): DEPRECATED. Field will be removed in future versions. Please use slug instead.
        description (str):
        authorization (ServerAuthorizationOutput):
        transport_config (HostedTransportConfigOutput | HttpStreamingTransportConfigOutput | Schema57Type3 |
            SseTransportConfigOutput | StdioTransportConfigOutput):
        oauth_client_configuration (None | ServerOAuthClientConfiguration):
        oauth_metadata (None | ServerOAuthMetadata):
        is_enabled (bool):
        last_tool_refresh_at (None | str):
        created_at (str):
        updated_at (str):
        logo_url (str | Unset):
    """

    id: float
    slug: str
    tenant_id: str
    name: str
    description: str
    authorization: ServerAuthorizationOutput
    transport_config: (
        HostedTransportConfigOutput
        | HttpStreamingTransportConfigOutput
        | Schema57Type3
        | SseTransportConfigOutput
        | StdioTransportConfigOutput
    )
    oauth_client_configuration: None | ServerOAuthClientConfiguration
    oauth_metadata: None | ServerOAuthMetadata
    is_enabled: bool
    last_tool_refresh_at: None | str
    created_at: str
    updated_at: str
    logo_url: str | Unset = UNSET

    def to_dict(self) -> dict[str, Any]:
        from ..models.http_streaming_transport_config_output import HttpStreamingTransportConfigOutput
        from ..models.schema_57_type_3 import Schema57Type3
        from ..models.server_o_auth_client_configuration import ServerOAuthClientConfiguration
        from ..models.server_o_auth_metadata import ServerOAuthMetadata
        from ..models.sse_transport_config_output import SseTransportConfigOutput
        from ..models.stdio_transport_config_output import StdioTransportConfigOutput

        id = self.id

        slug = self.slug

        tenant_id = self.tenant_id

        name = self.name

        description = self.description

        authorization = self.authorization.to_dict()

        transport_config: dict[str, Any]
        if isinstance(self.transport_config, HttpStreamingTransportConfigOutput):
            transport_config = self.transport_config.to_dict()
        elif isinstance(self.transport_config, StdioTransportConfigOutput):
            transport_config = self.transport_config.to_dict()
        elif isinstance(self.transport_config, SseTransportConfigOutput):
            transport_config = self.transport_config.to_dict()
        elif isinstance(self.transport_config, Schema57Type3):
            transport_config = self.transport_config.to_dict()
        else:
            transport_config = self.transport_config.to_dict()

        oauth_client_configuration: dict[str, Any] | None
        if isinstance(self.oauth_client_configuration, ServerOAuthClientConfiguration):
            oauth_client_configuration = self.oauth_client_configuration.to_dict()
        else:
            oauth_client_configuration = self.oauth_client_configuration

        oauth_metadata: dict[str, Any] | None
        if isinstance(self.oauth_metadata, ServerOAuthMetadata):
            oauth_metadata = self.oauth_metadata.to_dict()
        else:
            oauth_metadata = self.oauth_metadata

        is_enabled = self.is_enabled

        last_tool_refresh_at: None | str
        last_tool_refresh_at = self.last_tool_refresh_at

        created_at = self.created_at

        updated_at = self.updated_at

        logo_url = self.logo_url

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "slug": slug,
                "tenantId": tenant_id,
                "name": name,
                "description": description,
                "authorization": authorization,
                "transportConfig": transport_config,
                "oauthClientConfiguration": oauth_client_configuration,
                "oauthMetadata": oauth_metadata,
                "isEnabled": is_enabled,
                "lastToolRefreshAt": last_tool_refresh_at,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )
        if logo_url is not UNSET:
            field_dict["logoUrl"] = logo_url

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.hosted_transport_config_output import HostedTransportConfigOutput
        from ..models.http_streaming_transport_config_output import HttpStreamingTransportConfigOutput
        from ..models.schema_57_type_3 import Schema57Type3
        from ..models.server_authorization_output import ServerAuthorizationOutput
        from ..models.server_o_auth_client_configuration import ServerOAuthClientConfiguration
        from ..models.server_o_auth_metadata import ServerOAuthMetadata
        from ..models.sse_transport_config_output import SseTransportConfigOutput
        from ..models.stdio_transport_config_output import StdioTransportConfigOutput

        d = dict(src_dict)
        id = d.pop("id")

        slug = d.pop("slug")

        tenant_id = d.pop("tenantId")

        name = d.pop("name")

        description = d.pop("description")

        authorization = ServerAuthorizationOutput.from_dict(d.pop("authorization"))

        def _parse_transport_config(
            data: object,
        ) -> (
            HostedTransportConfigOutput
            | HttpStreamingTransportConfigOutput
            | Schema57Type3
            | SseTransportConfigOutput
            | StdioTransportConfigOutput
        ):
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_schema57_type_0 = HttpStreamingTransportConfigOutput.from_dict(data)

                return componentsschemas_schema57_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_schema57_type_1 = StdioTransportConfigOutput.from_dict(data)

                return componentsschemas_schema57_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_schema57_type_2 = SseTransportConfigOutput.from_dict(data)

                return componentsschemas_schema57_type_2
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_schema57_type_3 = Schema57Type3.from_dict(data)

                return componentsschemas_schema57_type_3
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_schema57_type_4 = HostedTransportConfigOutput.from_dict(data)

            return componentsschemas_schema57_type_4

        transport_config = _parse_transport_config(d.pop("transportConfig"))

        def _parse_oauth_client_configuration(data: object) -> None | ServerOAuthClientConfiguration:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_schema58_type_1 = ServerOAuthClientConfiguration.from_dict(data)

                return componentsschemas_schema58_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ServerOAuthClientConfiguration, data)

        oauth_client_configuration = _parse_oauth_client_configuration(d.pop("oauthClientConfiguration"))

        def _parse_oauth_metadata(data: object) -> None | ServerOAuthMetadata:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_schema59_type_1 = ServerOAuthMetadata.from_dict(data)

                return componentsschemas_schema59_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ServerOAuthMetadata, data)

        oauth_metadata = _parse_oauth_metadata(d.pop("oauthMetadata"))

        is_enabled = d.pop("isEnabled")

        def _parse_last_tool_refresh_at(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        last_tool_refresh_at = _parse_last_tool_refresh_at(d.pop("lastToolRefreshAt"))

        created_at = d.pop("createdAt")

        updated_at = d.pop("updatedAt")

        logo_url = d.pop("logoUrl", UNSET)

        server_dto = cls(
            id=id,
            slug=slug,
            tenant_id=tenant_id,
            name=name,
            description=description,
            authorization=authorization,
            transport_config=transport_config,
            oauth_client_configuration=oauth_client_configuration,
            oauth_metadata=oauth_metadata,
            is_enabled=is_enabled,
            last_tool_refresh_at=last_tool_refresh_at,
            created_at=created_at,
            updated_at=updated_at,
            logo_url=logo_url,
        )

        return server_dto



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.get_mcp_servers_server_slug_credentials_authorize_url_redirect import GetMcpServersServerSlugCredentialsAuthorizeUrlRedirect
from ...models.get_mcp_servers_server_slug_credentials_authorize_url_response_200 import GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200
from ...models.get_mcp_servers_server_slug_credentials_authorize_url_return_to import GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo
from ...models.schema_32 import Schema32
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    server_slug: str,
    *,
    scope: Schema32 | Unset = UNSET,
    redirect: GetMcpServersServerSlugCredentialsAuthorizeUrlRedirect | Unset = UNSET,
    return_to: GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo | Unset = UNSET,
    profile_id: str | Unset = UNSET,
    user_sub: str | Unset = UNSET,

) -> dict[str, Any]:
    

    

    params: dict[str, Any] = {}

    json_scope: str | Unset = UNSET
    if not isinstance(scope, Unset):
        json_scope = scope.value

    params["scope"] = json_scope

    json_redirect: str | Unset = UNSET
    if not isinstance(redirect, Unset):
        json_redirect = redirect.value

    params["redirect"] = json_redirect

    json_return_to: str | Unset = UNSET
    if not isinstance(return_to, Unset):
        json_return_to = return_to.value

    params["returnTo"] = json_return_to

    params["profileId"] = profile_id

    params["userSub"] = user_sub


    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}


    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/mcp-servers/{server_slug}/credentials/authorize-url".format(server_slug=quote(str(server_slug), safe=""),),
        "params": params,
    }


    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200 | None:
    if response.status_code == 200:
        response_200 = GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200.from_dict(response.json())



        return response_200

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    server_slug: str,
    *,
    client: AuthenticatedClient | Client,
    scope: Schema32 | Unset = UNSET,
    redirect: GetMcpServersServerSlugCredentialsAuthorizeUrlRedirect | Unset = UNSET,
    return_to: GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo | Unset = UNSET,
    profile_id: str | Unset = UNSET,
    user_sub: str | Unset = UNSET,

) -> Response[GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200]:
    """  Get an out-of-band OAuth URL which can be opened in any browser on any device to provide
    authorization credentials for this request's authenticated user. The generated URL will not require
    user to sign-in to Gatana; instead it will directly start the OAuth authorization flow with the
    remote MCP server.

    Args:
        server_slug (str):
        scope (Schema32 | Unset):
        redirect (GetMcpServersServerSlugCredentialsAuthorizeUrlRedirect | Unset):
        return_to (GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo | Unset):
        profile_id (str | Unset):
        user_sub (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200]
     """


    kwargs = _get_kwargs(
        server_slug=server_slug,
scope=scope,
redirect=redirect,
return_to=return_to,
profile_id=profile_id,
user_sub=user_sub,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    server_slug: str,
    *,
    client: AuthenticatedClient | Client,
    scope: Schema32 | Unset = UNSET,
    redirect: GetMcpServersServerSlugCredentialsAuthorizeUrlRedirect | Unset = UNSET,
    return_to: GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo | Unset = UNSET,
    profile_id: str | Unset = UNSET,
    user_sub: str | Unset = UNSET,

) -> GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200 | None:
    """  Get an out-of-band OAuth URL which can be opened in any browser on any device to provide
    authorization credentials for this request's authenticated user. The generated URL will not require
    user to sign-in to Gatana; instead it will directly start the OAuth authorization flow with the
    remote MCP server.

    Args:
        server_slug (str):
        scope (Schema32 | Unset):
        redirect (GetMcpServersServerSlugCredentialsAuthorizeUrlRedirect | Unset):
        return_to (GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo | Unset):
        profile_id (str | Unset):
        user_sub (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200
     """


    return sync_detailed(
        server_slug=server_slug,
client=client,
scope=scope,
redirect=redirect,
return_to=return_to,
profile_id=profile_id,
user_sub=user_sub,

    ).parsed

async def asyncio_detailed(
    server_slug: str,
    *,
    client: AuthenticatedClient | Client,
    scope: Schema32 | Unset = UNSET,
    redirect: GetMcpServersServerSlugCredentialsAuthorizeUrlRedirect | Unset = UNSET,
    return_to: GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo | Unset = UNSET,
    profile_id: str | Unset = UNSET,
    user_sub: str | Unset = UNSET,

) -> Response[GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200]:
    """  Get an out-of-band OAuth URL which can be opened in any browser on any device to provide
    authorization credentials for this request's authenticated user. The generated URL will not require
    user to sign-in to Gatana; instead it will directly start the OAuth authorization flow with the
    remote MCP server.

    Args:
        server_slug (str):
        scope (Schema32 | Unset):
        redirect (GetMcpServersServerSlugCredentialsAuthorizeUrlRedirect | Unset):
        return_to (GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo | Unset):
        profile_id (str | Unset):
        user_sub (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200]
     """


    kwargs = _get_kwargs(
        server_slug=server_slug,
scope=scope,
redirect=redirect,
return_to=return_to,
profile_id=profile_id,
user_sub=user_sub,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    server_slug: str,
    *,
    client: AuthenticatedClient | Client,
    scope: Schema32 | Unset = UNSET,
    redirect: GetMcpServersServerSlugCredentialsAuthorizeUrlRedirect | Unset = UNSET,
    return_to: GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo | Unset = UNSET,
    profile_id: str | Unset = UNSET,
    user_sub: str | Unset = UNSET,

) -> GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200 | None:
    """  Get an out-of-band OAuth URL which can be opened in any browser on any device to provide
    authorization credentials for this request's authenticated user. The generated URL will not require
    user to sign-in to Gatana; instead it will directly start the OAuth authorization flow with the
    remote MCP server.

    Args:
        server_slug (str):
        scope (Schema32 | Unset):
        redirect (GetMcpServersServerSlugCredentialsAuthorizeUrlRedirect | Unset):
        return_to (GetMcpServersServerSlugCredentialsAuthorizeUrlReturnTo | Unset):
        profile_id (str | Unset):
        user_sub (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        GetMcpServersServerSlugCredentialsAuthorizeUrlResponse200
     """


    return (await asyncio_detailed(
        server_slug=server_slug,
client=client,
scope=scope,
redirect=redirect,
return_to=return_to,
profile_id=profile_id,
user_sub=user_sub,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define

from ..models.schema_67 import Schema67

if TYPE_CHECKING:
    from ..models.server_authorization_output import ServerAuthorizationOutput
    from ..models.server_credentials_dto import ServerCredentialsDto


T = TypeVar("T", bound="ProfileDetailsDtoServersItem")


@_attrs_define
class ProfileDetailsDtoServersItem:
    """
    Attributes:
        name (str):
        slug (str):
        authorization (ServerAuthorizationOutput):
        credentials_num_keys (float | None):
        credential_scope (Schema67):
        credentials (None | ServerCredentialsDto):
    """

    name: str
    slug: str
    authorization: ServerAuthorizationOutput
    credentials_num_keys: float | None
    credential_scope: Schema67
    credentials: None | ServerCredentialsDto

    def to_dict(self) -> dict[str, Any]:
        from ..models.server_credentials_dto import ServerCredentialsDto

        name = self.name

        slug = self.slug

        authorization = self.authorization.to_dict()

        credentials_num_keys: float | None
        credentials_num_keys = self.credentials_num_keys

        credential_scope = self.credential_scope.value

        credentials: dict[str, Any] | None
        if isinstance(self.credentials, ServerCredentialsDto):
            credentials = self.credentials.to_dict()
        else:
            credentials = self.credentials

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "name": name,
                "slug": slug,
                "authorization": authorization,
                "credentialsNumKeys": credentials_num_keys,
                "credentialScope": credential_scope,
                "credentials": credentials,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.server_authorization_output import ServerAuthorizationOutput
        from ..models.server_credentials_dto import ServerCredentialsDto

        d = dict(src_dict)
        name = d.pop("name")

        slug = d.pop("slug")

        authorization = ServerAuthorizationOutput.from_dict(d.pop("authorization"))

        def _parse_credentials_num_keys(data: object) -> float | None:
            if data is None:
                return data
            return cast(float | None, data)

        credentials_num_keys = _parse_credentials_num_keys(d.pop("credentialsNumKeys"))

        credential_scope = Schema67(d.pop("credentialScope"))

        def _parse_credentials(data: object) -> None | ServerCredentialsDto:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                credentials_type_0 = ServerCredentialsDto.from_dict(data)

                return credentials_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ServerCredentialsDto, data)

        credentials = _parse_credentials(d.pop("credentials"))

        profile_details_dto_servers_item = cls(
            name=name,
            slug=slug,
            authorization=authorization,
            credentials_num_keys=credentials_num_keys,
            credential_scope=credential_scope,
            credentials=credentials,
        )

        return profile_details_dto_servers_item



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.server_o_auth_client_configuration_grant_type import ServerOAuthClientConfigurationGrantType
from typing import cast
from typing import Literal, cast






T = TypeVar("T", bound="ServerOAuthClientConfiguration")



@_attrs_define
class ServerOAuthClientConfiguration:
    """ 
        Attributes:
            client_id (str):  Default: ''.
            client_secret (str):  Default: ''.
            grant_type (ServerOAuthClientConfigurationGrantType):
            client_auth_method (Literal['client_secret_basic'] | Literal['client_secret_post'] | Literal['none'] | str):
            scopes (str):  Default: ''.
     """

    grant_type: ServerOAuthClientConfigurationGrantType
    client_auth_method: Literal['client_secret_basic'] | Literal['client_secret_post'] | Literal['none'] | str
    client_id: str = ''
    client_secret: str = ''
    scopes: str = ''





    def to_dict(self) -> dict[str, Any]:
        client_id = self.client_id

        client_secret = self.client_secret

        grant_type = self.grant_type.value

        client_auth_method: Literal['client_secret_basic'] | Literal['client_secret_post'] | Literal['none'] | str
        client_auth_method = self.client_auth_method

        scopes = self.scopes


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "clientId": client_id,
            "clientSecret": client_secret,
            "grantType": grant_type,
            "clientAuthMethod": client_auth_method,
            "scopes": scopes,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        client_id = d.pop("clientId")

        client_secret = d.pop("clientSecret")

        grant_type = ServerOAuthClientConfigurationGrantType(d.pop("grantType"))




        def _parse_client_auth_method(data: object) -> Literal['client_secret_basic'] | Literal['client_secret_post'] | Literal['none'] | str:
            client_auth_method_type_0 = cast(Literal['client_secret_basic'] , data)
            if client_auth_method_type_0 != 'client_secret_basic':
                raise ValueError(f"clientAuthMethod_type_0 must match const 'client_secret_basic', got '{client_auth_method_type_0}'")
            return client_auth_method_type_0
            client_auth_method_type_1 = cast(Literal['client_secret_post'] , data)
            if client_auth_method_type_1 != 'client_secret_post':
                raise ValueError(f"clientAuthMethod_type_1 must match const 'client_secret_post', got '{client_auth_method_type_1}'")
            return client_auth_method_type_1
            client_auth_method_type_2 = cast(Literal['none'] , data)
            if client_auth_method_type_2 != 'none':
                raise ValueError(f"clientAuthMethod_type_2 must match const 'none', got '{client_auth_method_type_2}'")
            return client_auth_method_type_2
            return cast(Literal['client_secret_basic'] | Literal['client_secret_post'] | Literal['none'] | str, data)

        client_auth_method = _parse_client_auth_method(d.pop("clientAuthMethod"))


        scopes = d.pop("scopes")

        server_o_auth_client_configuration = cls(
            client_id=client_id,
            client_secret=client_secret,
            grant_type=grant_type,
            client_auth_method=client_auth_method,
            scopes=scopes,
        )

        return server_o_auth_client_configuration




from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.put_mcp_servers_server_slug_config_body_type_0 import PutMcpServersServerSlugConfigBodyType0
from ...models.put_mcp_servers_server_slug_config_body_type_1 import PutMcpServersServerSlugConfigBodyType1
from ...models.put_mcp_servers_server_slug_config_body_type_2 import PutMcpServersServerSlugConfigBodyType2
from ...models.put_mcp_servers_server_slug_config_body_type_3 import PutMcpServersServerSlugConfigBodyType3
from ...models.put_mcp_servers_server_slug_config_body_type_4 import PutMcpServersServerSlugConfigBodyType4
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    server_slug: str,
    *,
    body: PutMcpServersServerSlugConfigBodyType0 | PutMcpServersServerSlugConfigBodyType1 | PutMcpServersServerSlugConfigBodyType2 | PutMcpServersServerSlugConfigBodyType3 | PutMcpServersServerSlugConfigBodyType4 | Unset = UNSET,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "put",
        "url": "/mcp-servers/{server_slug}/config".format(server_slug=quote(str(server_slug), safe=""),),
    }

    
    if isinstance(body, PutMcpServersServerSlugConfigBodyType0):
        _kwargs["json"] = body.to_dict()
    elif isinstance(body, PutMcpServersServerSlugConfigBodyType1):
        _kwargs["json"] = body.to_dict()
    elif isinstance(body, PutMcpServersServerSlugConfigBodyType2):
        _kwargs["json"] = body.to_dict()
    elif isinstance(body, PutMcpServersServerSlugConfigBodyType3):
        _kwargs["json"] = body.to_dict()
    else:
        _kwargs["json"] = body.to_dict()



    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Any | None:
    if response.status_code == 200:
        return None

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    server_slug: str,
    *,
    client: AuthenticatedClient | Client,
    body: PutMcpServersServerSlugConfigBodyType0 | PutMcpServersServerSlugConfigBodyType1 | PutMcpServersServerSlugConfigBodyType2 | PutMcpServersServerSlugConfigBodyType3 | PutMcpServersServerSlugConfigBodyType4 | Unset = UNSET,

) -> Response[Any]:
    """ 
    Args:
        server_slug (str):
        body (PutMcpServersServerSlugConfigBodyType0 | PutMcpServersServerSlugConfigBodyType1 |
            PutMcpServersServerSlugConfigBodyType2 | PutMcpServersServerSlugConfigBodyType3 |
            PutMcpServersServerSlugConfigBodyType4 | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
     """


    kwargs = _get_kwargs(
        server_slug=server_slug,
body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    server_slug: str,
    *,
    client: AuthenticatedClient | Client,
    body: PutMcpServersServerSlugConfigBodyType0 | PutMcpServersServerSlugConfigBodyType1 | PutMcpServersServerSlugConfigBodyType2 | PutMcpServersServerSlugConfigBodyType3 | PutMcpServersServerSlugConfigBodyType4 | Unset = UNSET,

) -> Response[Any]:
    """ 
    Args:
        server_slug (str):
        body (PutMcpServersServerSlugConfigBodyType0 | PutMcpServersServerSlugConfigBodyType1 |
            PutMcpServersServerSlugConfigBodyType2 | PutMcpServersServerSlugConfigBodyType3 |
            PutMcpServersServerSlugConfigBodyType4 | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
     """


    kwargs = _get_kwargs(
        server_slug=server_slug,
body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.personal_access_token import PersonalAccessToken





T = TypeVar("T", bound="GetUsersUserSubPersonalAccessTokensResponse200")



@_attrs_define
class GetUsersUserSubPersonalAccessTokensResponse200:
    """ 
        Attributes:
            tokens (list[PersonalAccessToken]):
     """

    tokens: list[PersonalAccessToken]





    def to_dict(self) -> dict[str, Any]:
        from ..models.personal_access_token import PersonalAccessToken
        tokens = []
        for tokens_item_data in self.tokens:
            tokens_item = tokens_item_data.to_dict()
            tokens.append(tokens_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "tokens": tokens,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.personal_access_token import PersonalAccessToken
        d = dict(src_dict)
        tokens = []
        _tokens = d.pop("tokens")
        for tokens_item_data in (_tokens):
            tokens_item = PersonalAccessToken.from_dict(tokens_item_data)



            tokens.append(tokens_item)


        get_users_user_sub_personal_access_tokens_response_200 = cls(
            tokens=tokens,
        )

        return get_users_user_sub_personal_access_tokens_response_200




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.profile import Profile





T = TypeVar("T", bound="GetProfilesResponse200")



@_attrs_define
class GetProfilesResponse200:
    """ 
        Attributes:
            profiles (list[Profile]):
     """

    profiles: list[Profile]





    def to_dict(self) -> dict[str, Any]:
        from ..models.profile import Profile
        profiles = []
        for profiles_item_data in self.profiles:
            profiles_item = profiles_item_data.to_dict()
            profiles.append(profiles_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "profiles": profiles,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.profile import Profile
        d = dict(src_dict)
        profiles = []
        _profiles = d.pop("profiles")
        for profiles_item_data in (_profiles):
            profiles_item = Profile.from_dict(profiles_item_data)



            profiles.append(profiles_item)


        get_profiles_response_200 = cls(
            profiles=profiles,
        )

        return get_profiles_response_200




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Literal, cast






T = TypeVar("T", bound="AzureKeyVaultConfiguration")



@_attrs_define
class AzureKeyVaultConfiguration:
    """ 
        Attributes:
            type_ (Literal['azure_key_vault']):
            vault_url (str):
            tenant_id (str):
            client_id (str):
            client_secret (str):
     """

    type_: Literal['azure_key_vault']
    vault_url: str
    tenant_id: str
    client_id: str
    client_secret: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        vault_url = self.vault_url

        tenant_id = self.tenant_id

        client_id = self.client_id

        client_secret = self.client_secret


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "vaultUrl": vault_url,
            "tenantId": tenant_id,
            "clientId": client_id,
            "clientSecret": client_secret,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = cast(Literal['azure_key_vault'] , d.pop("type"))
        if type_ != 'azure_key_vault':
            raise ValueError(f"type must match const 'azure_key_vault', got '{type_}'")

        vault_url = d.pop("vaultUrl")

        tenant_id = d.pop("tenantId")

        client_id = d.pop("clientId")

        client_secret = d.pop("clientSecret")

        azure_key_vault_configuration = cls(
            type_=type_,
            vault_url=vault_url,
            tenant_id=tenant_id,
            client_id=client_id,
            client_secret=client_secret,
        )


        azure_key_vault_configuration.additional_properties = d
        return azure_key_vault_configuration

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum

class TenantOidcConfigurationClientAuthMethod(str, Enum):
    CLIENT_SECRET_BASIC = "client_secret_basic"
    CLIENT_SECRET_POST = "client_secret_post"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define

from ..models.schema_66 import Schema66

T = TypeVar("T", bound="GetMembersResponseTeamsItem")


@_attrs_define
class GetMembersResponseTeamsItem:
    """
    Attributes:
        role (Schema66):
        id (str):
        name (str):
    """

    role: Schema66
    id: str
    name: str

    def to_dict(self) -> dict[str, Any]:
        role = self.role.value

        id = self.id

        name = self.name

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "role": role,
                "id": id,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        role = Schema66(d.pop("role"))

        id = d.pop("id")

        name = d.pop("name")

        get_members_response_teams_item = cls(
            role=role,
            id=id,
            name=name,
        )

        return get_members_response_teams_item
